Package: PStrata
Type: Package
Title: Principal Stratification Analysis in R
Version: 1.0.1
Date: 2026-05-15
Encoding: UTF-8
Authors@R: c(
    person(given = "Bo", family = "Liu", role = c("aut", "cre"), email = "bo.liu1997@gmail.com"),
    person(given = "Fan", family = "Li", role = "ctb", email = "fl35@duke.edu"))
Author: Bo Liu [aut, cre],
  Fan Li [ctb]
Maintainer: Bo Liu <bo.liu1997@gmail.com>
Description: Estimating causal effects in the presence of post-treatment confounding
    using principal stratification. 'PStrata' allows for customized monotonicity
    assumptions and exclusion restriction assumptions, with automatic full Bayesian
    inference supported by 'Stan'. The main workflow is PStrataModel() to specify the
    model, fit() to run MCMC sampling, estimate() to extract potential outcomes, and
    contrast() to compute causal effects. Visualization tools are provided for
    diagnosis and interpretation. See Liu and Li (2023)
    <doi:10.48550/arXiv.2304.02740> for details.
URL: https://github.com/LauBok/PStrata
BugReports: https://github.com/LauBok/PStrata/issues
Depends: R (>= 3.5.0)
Collate: PStrata-package.R utils.R survival.R prior.R make_stancode.R
        sim_data_normal.R sim_data_Cox.R PStrataModel.R fit.R
        estimate.R contrast.R
License: GPL (>= 2)
Suggests: R.rsp
Imports: ggplot2, rstan, lme4, reformulas, purrr, stringr, stats
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-15 12:23:21 UTC; boliu
Repository: CRAN
Date/Publication: 2026-05-15 15:50:20 UTC
Built: R 4.6.0; ; 2026-05-15 18:29:15 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",PSEstimate)
S3method(coef,PStrataFit)
S3method(contrast,PSEstimate)
S3method(contrast,PStrataFit)
S3method(diagnostics,PStrataFit)
S3method(estimate,PStrataFit)
S3method(fit,PStrataModel)
S3method(formula,PStrataFit)
S3method(nobs,PStrataFit)
S3method(plot,PSContrast)
S3method(plot,PSEstimate)
S3method(plot,PStrataFit)
S3method(print,PSContrast)
S3method(print,PSEstimate)
S3method(print,PStrataFit)
S3method(print,PStrataFitSurvival)
S3method(print,PStrataModel)
S3method(print,summary.PStrataFit)
S3method(summary,PSEstimate)
S3method(summary,PStrataFit)
S3method(summary,PStrataModel)
export(PStrataModel)
export(contrast)
export(diagnostics)
export(estimate)
export(fit)
export(make_stancode)
export(prior_cauchy)
export(prior_chisq)
export(prior_exponential)
export(prior_flat)
export(prior_gamma)
export(prior_inv_chisq)
export(prior_inv_gamma)
export(prior_lasso)
export(prior_logistic)
export(prior_normal)
export(prior_t)
export(prior_weibull)
export(stancode.PStrataFit)
export(survival)
importFrom(stats,coef)
importFrom(stats,formula)
importFrom(stats,nobs)

# PStrata

<!-- badges: start -->
<!-- badges: end -->

**PStrata** fits Bayesian principal stratification models using **Stan**. It
supports a wide variety of outcome families, link functions, priors, and
customizable monotonicity / exclusion-restriction assumptions for causal
inference in the presence of post-treatment confounding.

See Liu and Li (2023) [arXiv:2304.02740](https://arxiv.org/abs/2304.02740) for
methodological details.

## Installation

```r
# Install from GitHub
devtools::install_github("LauBok/PStrata")
```

**PStrata** compiles models with **Stan** via `rstan`, so a working C++
toolchain is required (Rtools on Windows, Xcode command-line tools on macOS).
See the
[RStan Getting Started](https://github.com/stan-dev/rstan/wiki/RStan-Getting-Started)
guide.

## Overview

Let `Z` denote the assigned treatment, `D` the post-randomization intermediate
variable, `X` the covariates and `Y` the outcome. The principal stratum is
`S = (D(0), D(1))`. **PStrata** specifies two models:

- **S-model** — a multinomial (softmax) model for the principal stratum given
  covariates: `log p(S=s | X) / p(S=s0 | X) = X beta^S`.
- **Y-model** — a generalized linear (or survival) model for the outcome,
  specific to a principal stratum and treatment arm:
  `E[Y | X, S, Z] = g^{-1}(X beta_{s,z}^Y)`.

### Assumptions

- **Monotonicity** — you explicitly list every principal stratum assumed to
  exist via the `strata` argument; any stratum you omit (e.g. defiers `"10"`)
  is ruled out. This is more flexible than a fixed monotonicity assumption.
- **Exclusion restriction (ER)** — for a stratum where `D(0) = D(1)`, the
  outcome distribution does not depend on `Z`. Strata under ER are named in
  the `ER` argument.

## Workflow

| Step | Function | Purpose |
|---|---|---|
| 1 | `PStrataModel()` | Specify the model symbolically (no data needed) |
| 2 | `fit()` | Build data, generate Stan code, run MCMC |
| 3 | `estimate()` | Extract posterior potential outcomes per stratum × arm |
| 4 | `contrast()` | Compute causal effects (e.g. `Y(1) - Y(0)`) |

`print()`, `summary()`, `plot()`, `diagnostics()`, and `stancode()` are
available on the fitted object.

## Example

### Normal outcome

Fit an intercept-only model on `sim_data_normal` (non-compliance: never-takers,
compliers, always-takers) under monotonicity, with exclusion restriction on
never-takers and always-takers.

```r
library(PStrata)

model <- PStrataModel(
  S.formula = Z + D ~ 1,
  Y.formula = Y ~ 1,
  Y.family  = gaussian(link = "identity"),
  strata    = c(n = "00", c = "01", a = "11"),
  ER        = c("n", "a"),
  prior_intercept = prior_normal(0, 1),
  prior_sigma     = prior_inv_gamma(1)
)
summary(model)   # algebraic description of the specified model

ps_fit <- fit(model, data = sim_data_normal,
              chains = 4, warmup = 500, iter = 1000)
ps_fit            # stratum proportions + mean outcome per group
summary(ps_fit)   # posterior intervals for all parameters
diagnostics(ps_fit)
```

Each stratum is a string of `D` values, one digit per treatment arm: `"00"` =
never-taker, `"01"` = complier, `"11"` = always-taker. Names (`n`, `c`, `a`)
are optional. Multiple post-randomization variables use `|` (e.g. `"00|01"`)
or the list-of-lists form.

### Potential outcomes and contrasts

```r
est <- estimate(ps_fit)        # E[Y(z) | S = s] by stratum and arm
summary(est, "data.frame")
plot(est)

ctr <- contrast(ps_fit, Z = TRUE)   # treatment effect Y(1) - Y(0) per stratum
summary(ctr, "data.frame")
plot(ctr)
```

### Survival outcome

Time-to-event outcomes use `Y.family = survival("Cox")` (Weibull-Cox) or
`survival("AFT")` (log-normal AFT). The outcome formula carries the event
indicator: `time + status ~ X`.

```r
model_s <- PStrataModel(
  S.formula = Z + D ~ 1,
  Y.formula = Y + delta ~ 1,
  Y.family  = survival("Cox"),
  strata    = c(n = "00", c = "01", a = "11"),
  ER        = c("n", "a"),
  prior_intercept = prior_normal(0, 1)
)
ps_fit_s <- fit(model_s, data = sim_data_Cox,
                chains = 4, warmup = 500, iter = 1000)

# Survival probability (or type = "RACE" for restricted average causal effect)
est_s <- estimate(ps_fit_s, type = "probability")
plot(est_s)

ctr_s <- contrast(ps_fit_s, Z = TRUE)
plot(ctr_s)
```

## Outcome families

`gaussian`, `binomial`, `Gamma`, `poisson`, `inverse.gaussian` (standard
`stats::family` objects with their usual links), plus `survival("Cox")` and
`survival("AFT")`.

> Binary-outcome principal stratification is weakly identified at small sample
> sizes / short chains; use enough data and MCMC iterations (e.g.
> `chains = 4`, `iter = 2000`) and check `diagnostics()` for convergence.

## Prior distributions

`prior_normal()`, `prior_t()`, `prior_cauchy()`, `prior_logistic()`,
`prior_lasso()`, `prior_exponential()`, `prior_gamma()`, `prior_inv_gamma()`,
`prior_chisq()`, `prior_inv_chisq()`, `prior_weibull()`, `prior_flat()`.

Assign them through the `PStrataModel()` arguments `prior_intercept`,
`prior_coefficient`, `prior_sigma`, `prior_alpha`, `prior_lambda`,
`prior_theta`.

## Key functions

| Function | Description |
|---|---|
| `PStrataModel()` | Specify a principal stratification model |
| `fit()` | Fit the model via Stan MCMC |
| `estimate()` | Posterior potential outcomes by stratum × arm |
| `contrast()` | Causal-effect contrasts (strata, arms, time) |
| `diagnostics()` | MCMC convergence diagnostics |
| `make_stancode()` / `stancode()` | Inspect the generated Stan program |
| `survival()` | Family constructor for time-to-event outcomes |

## Datasets

`sim_data_normal` (Gaussian outcome, non-compliance) and `sim_data_Cox`
(survival outcome) are bundled for illustration.

## License

GPL (>= 2)

#' Family Function for Survival Data
#'
#' Construct a family object for survival data
#'
#' @param method the parametric method used for survival data. Can be Cox or AFT.
#' @param link a link function, currently only identity is implemented and used
#' @return A \code{family} object
#' @examples
#' survival("Cox")
#' survival("AFT")
#' @export
survival <- function(method = c("Cox", "AFT"), link = "identity") {
  method <- match.arg(method)
  family <- paste0("survival_", method)
  structure(list(family = family, link = link), class = "family")
}

#' Simulated Dataset for Survival Outcome (Cox Model)
#'
#' A dataset generated for illustration of the principal stratification analysis.
#' This dataset represents the common case of non-compliance.
#' 
#' @details 
#' The dataset represents the scenario where actual treatment might not be in compliance
#' with the randomized (assigned) treatment. Defiers and always-takers are ruled out, leaving two
#' strata, "never-taker" and "complier" randomly sampled with 
#' probability 0.3, 0.7 respectively. The assigned treatment \eqn{Z} is randomized
#' with 0.5 probability for either arm. The true event time \eqn{T} is given by the following 
#' Weibull-Cox distribution
#' \describe{
#' \item{never-taker}{\eqn{Y \sim Weibull-Cox(theta = 1, mu = 0.3)}}
#' \item{complier}{\eqn{Y \sim Weibull-Cox(theta = 1, mu = 2 - 0.6*Z)}}
#' } and the censor time \eqn{C} is uniformly drawn between 0.5 and 2.
#' 
#' The exclusion restriction assumption holds for never-takers in this
#' generated dataset.
#'
#' @format ## `sim_data_Cox`
#' A data frame with 1,000 rows and 7 columns:
#' \describe{
#'   \item{S}{Principal Strata: "never taker", "complier" or "always taker"}
#'   \item{Z}{Randomized treatment arm: 0 = control, 1 = treatment}
#'   \item{D}{Actual treatment arm: 0 = control, 1 = treatment}
#'   \item{T}{True outcome: event time}
#'   \item{C}{Censor time}
#'   \item{delta}{Event indicator. 1 means true outcome is observed; 0 means otherwise}
#'   \item{Y}{The observed event time or censor time}
#' }
#' @examples
#' data(sim_data_Cox)
#' head(sim_data_Cox)
#' table(sim_data_Cox$D, sim_data_Cox$Z)
"sim_data_Cox"

# ===========================================================================
# PStrataModel: Model specification for principal stratification
# ===========================================================================

# --- Family registry ---
# Maps family + link to outcome type, Stan functions, and display equations

.family_registry <- list(
  gaussian = list(
    outcome_type = "real",
    stan_func = "normal_lpdf",
    extra_params = list(list(name = "sigma", type = "positive")),
    links = list(
      identity = list(
        stan_link = "identity_fn",
        eq = "Y | g, X ~ Normal(eta_g, sigma_g)"
      ),
      log = list(
        stan_link = "exp",
        eq = "Y | g, X ~ Normal(exp(eta_g), sigma_g)"
      ),
      inverse = list(
        stan_link = "inv",
        eq = "Y | g, X ~ Normal(1 / eta_g, sigma_g)"
      )
    )
  ),
  binomial = list(
    outcome_type = "binary",
    stan_func = "bernoulli_logit_lpmf",
    extra_params = list(),
    links = list(
      logit = list(
        stan_link = "identity_fn",
        eq = "P(Y=1 | g, X) = logistic(eta_g)"
      ),
      probit = list(
        stan_link = "inv_Phi",
        eq = "P(Y=1 | g, X) = Phi(eta_g)"
      ),
      cauchit = list(
        stan_link = "cauchy_cdf",
        eq = "P(Y=1 | g, X) = cauchy_cdf(eta_g)"
      ),
      log = list(
        stan_link = "exp",
        eq = "P(Y=1 | g, X) = exp(eta_g)"
      ),
      cloglog = list(
        stan_link = "inv_cloglog",
        eq = "P(Y=1 | g, X) = 1 - exp(-exp(eta_g))"
      )
    )
  ),
  Gamma = list(
    outcome_type = "positive",
    stan_func = "gamma_lpdf",
    extra_params = list(list(name = "alpha", type = "positive")),
    links = list(
      inverse = list(
        stan_link = "inv",
        eq = "Y | g, X ~ Gamma(alpha_g, alpha_g * eta_g)"
      ),
      identity = list(
        stan_link = "identity_fn",
        eq = "Y | g, X ~ Gamma(alpha_g, alpha_g / eta_g)"
      ),
      log = list(
        stan_link = "exp",
        eq = "Y | g, X ~ Gamma(alpha_g, alpha_g / exp(eta_g))"
      )
    )
  ),
  poisson = list(
    outcome_type = "count",
    stan_func = "poisson_lpmf",
    extra_params = list(),
    links = list(
      log = list(
        stan_link = "exp",
        eq = "Y | g, X ~ Poisson(exp(eta_g))"
      ),
      identity = list(
        stan_link = "identity_fn",
        eq = "Y | g, X ~ Poisson(eta_g)"
      )
    )
  ),
  inverse.gaussian = list(
    outcome_type = "positive",
    stan_func = "inv_gaussian_lpdf",
    extra_params = list(list(name = "lambda", type = "positive")),
    links = list(
      "1/mu^2" = list(
        stan_link = "inv_sqrt",
        eq = "Y | g, X ~ InvGaussian(1 / sqrt(eta_g), lambda_g)"
      ),
      inverse = list(
        stan_link = "inv",
        eq = "Y | g, X ~ InvGaussian(1 / eta_g, lambda_g)"
      ),
      identity = list(
        stan_link = "identity_fn",
        eq = "Y | g, X ~ InvGaussian(eta_g, lambda_g)"
      ),
      log = list(
        stan_link = "exp",
        eq = "Y | g, X ~ InvGaussian(exp(eta_g), lambda_g)"
      )
    )
  ),
  survival_Cox = list(
    outcome_type = "survival",
    stan_func = "weibull_cox_lpdf",
    extra_params = list(list(name = "theta", type = "positive")),
    links = list(
      identity = list(
        stan_link = "identity_fn",
        eq = "h(t | g, X) = theta_g * t^(theta_g - 1) * exp(eta_g)"
      )
    )
  ),
  survival_AFT = list(
    outcome_type = "survival",
    stan_func = "lognormal_lpdf",
    extra_params = list(list(name = "sigma", type = "positive")),
    links = list(
      identity = list(
        stan_link = "identity_fn",
        eq = "log(T) | g, X ~ Normal(eta_g, sigma_g)"
      )
    )
  )
)

# ===========================================================================
# Constructor
# ===========================================================================

#' Define a Principal Stratification Model
#'
#' Creates a model specification for principal stratification analysis.
#' No data is required at this stage -- the specification is purely symbolic.
#' Use \code{\link{fit}} to estimate the model with data.
#'
#' @param S.formula formula for the stratum model (e.g., \code{Z + D ~ X1 + X2}).
#' @param Y.formula formula for the outcome model (e.g., \code{Y ~ X1 + X2}).
#' @param Y.family a \code{family} object (e.g., \code{gaussian()}, \code{survival("Cox")}).
#' @param strata strata definition: character vector, list of vectors, or list of lists.
#' @param ER exclusion restriction: character vector of strata names or logical vector.
#' @param prior_intercept,prior_coefficient,prior_sigma,prior_alpha,prior_lambda,prior_theta
#'   prior distributions for model parameters.
#' @param survival.time.points number of time points for survival outcomes.
#'
#' @return An object of class \code{PStrataModel}.
#' @examples
#' # Non-compliance with three strata (never-taker, complier, always-taker)
#' model <- PStrataModel(
#'   S.formula = Z + D ~ 1,
#'   Y.formula = Y ~ 1,
#'   Y.family  = gaussian(),
#'   strata    = c(n = "00", c = "01", a = "11"),
#'   ER        = c("n", "a")
#' )
#' print(model)
#' summary(model)
#'
#' \donttest{
#' # Fit the model (requires rstan and C++ compiler)
#' data(sim_data_normal)
#' ps_fit <- fit(model, data = sim_data_normal, chains = 2, iter = 500)
#' summary(ps_fit)
#' plot(ps_fit)
#'
#' # Extract potential outcomes and contrasts
#' est <- estimate(ps_fit)
#' summary(est)
#' plot(est)
#'
#' ctr <- contrast(ps_fit)
#' summary(ctr)
#' plot(ctr)
#' }
#' @export
PStrataModel <- function(
    S.formula,
    Y.formula,
    Y.family,
    strata,
    ER = NULL,
    prior_intercept = prior_flat(),
    prior_coefficient = prior_normal(),
    prior_sigma = prior_inv_gamma(),
    prior_alpha = prior_inv_gamma(),
    prior_lambda = prior_inv_gamma(),
    prior_theta = prior_normal(),
    survival.time.points = 50
) {
  cl <- match.call()

  # Parse formulas symbolically
  s_parsed <- parse_formula_symbolic(S.formula)
  y_parsed <- parse_formula_symbolic(Y.formula)

  treatment_name <- s_parsed$lhs_names[1]
  postrand_names <- s_parsed$lhs_names[-1]
  outcome_name   <- y_parsed$lhs_names[1]
  event_name     <- if (length(y_parsed$lhs_names) > 1) y_parsed$lhs_names[2] else NULL

  # Parse strata
  strata_parsed <- parse_strata_def(strata)

  # Resolve ER
  er_list <- resolve_ER_model(ER, strata_parsed$strata_names)

  # Cross-validate formula vs strata
  if (length(postrand_names) != strata_parsed$num_postrand) {
    stop(
      "Formula has ", length(postrand_names),
      " post-randomization variable(s) but strata defines ",
      strata_parsed$num_postrand, "."
    )
  }

  # Resolve family
  family_name <- Y.family$family
  family_link <- Y.family$link
  family_info <- resolve_family_info(family_name, family_link)

  is_survival <- family_info$outcome_type == "survival"

  if (is_survival && is.null(event_name)) {
    stop("Survival family requires an event indicator in Y.formula (e.g., Y + delta ~ ...).")
  }

  # Compute outcome groups for display
  outcome_groups <- compute_outcome_groups(
    strata_parsed$strata_names, strata_parsed$strata_def,
    er_list, strata_parsed$num_treatment
  )

  # Collect relevant priors
  priors <- list(intercept = prior_intercept, coefficient = prior_coefficient)
  for (ep in family_info$extra_params) {
    priors[[ep$name]] <- switch(ep$name,
      sigma  = prior_sigma,
      alpha  = prior_alpha,
      lambda = prior_lambda,
      theta  = prior_theta
    )
  }

  res <- list(
    call               = cl,
    stratum_formula     = S.formula,
    outcome_formula     = Y.formula,
    outcome_family      = Y.family,
    treatment_name      = treatment_name,
    postrand_names      = postrand_names,
    outcome_name        = outcome_name,
    event_name          = event_name,
    strata_names        = strata_parsed$strata_names,
    strata_def          = strata_parsed$strata_def,
    exclusion_restriction = er_list,
    num_strata          = length(strata_parsed$strata_names),
    num_treatment       = strata_parsed$num_treatment,
    num_postrand        = length(postrand_names),
    family_info         = family_info,
    priors              = priors,
    outcome_groups      = outcome_groups,
    is_survival         = is_survival,
    survival_time_points = if (is_survival) survival.time.points else NULL,
    s_random_effects    = s_parsed$random_effects,
    y_random_effects    = y_parsed$random_effects,
    s_terms             = s_parsed$rhs_terms,
    y_terms             = y_parsed$rhs_terms
  )

  class(res) <- if (is_survival) {
    c("PStrataModelSurvival", "PStrataModel")
  } else {
    "PStrataModel"
  }

  res
}

# ===========================================================================
# Symbolic formula parsing (no data needed)
# ===========================================================================

#' @noRd
parse_formula_symbolic <- function(formula) {
  lhs <- if (length(formula) == 3) formula[[2]] else NULL
  lhs_names <- extract_lhs_names(lhs)

  bars <- reformulas::findbars(formula)
  random_effects <- NULL

  if (!is.null(bars) && length(bars) > 0) {
    random_effects <- lapply(bars, function(bar) {
      group <- deparse(bar[[3]])
      terms_expr <- bar[[2]]
      terms_str <- deparse(terms_expr)

      tf <- stats::terms(stats::reformulate(terms_str))
      has_intercept <- attr(tf, "intercept") == 1L
      has_slopes <- length(attr(tf, "term.labels")) > 0

      list(
        formula_str  = paste0("(", terms_str, " | ", group, ")"),
        group        = group,
        has_intercept = has_intercept,
        has_slopes   = has_slopes
      )
    })
  }

  # Extract fixed-effect term labels from RHS
  fixed_formula <- reformulas::nobars(formula)
  rhs_terms <- if (length(fixed_formula) == 3) {
    tf <- stats::terms(fixed_formula)
    has_intercept <- attr(tf, "intercept") == 1L
    term_labels <- attr(tf, "term.labels")
    list(has_intercept = has_intercept, covariates = term_labels)
  } else {
    list(has_intercept = FALSE, covariates = character(0))
  }

  list(
    lhs_names      = lhs_names,
    rhs_terms      = rhs_terms,
    random_effects = random_effects
  )
}

#' Recursively extract symbol names from a formula LHS
#' @noRd
extract_lhs_names <- function(node) {
  if (is.null(node)) return(character(0))
  if (is.name(node)) return(as.character(node))
  if (is.call(node)) {
    return(unlist(unique(lapply(node[-1], extract_lhs_names))))
  }
  character(0)
}

# ===========================================================================
# Strata parsing (3 input forms)
# ===========================================================================

#' @noRd
parse_strata_def <- function(strata) {
  if (is.character(strata)) {
    parse_strata_string(strata)
  } else if (is.list(strata)) {
    first <- strata[[1]]
    if (is.list(first)) {
      parse_strata_list_of_lists(strata)
    } else {
      parse_strata_list_of_vectors(strata)
    }
  } else {
    stop("strata must be a character vector or a named list.")
  }
}

#' @noRd
parse_strata_string <- function(strata) {
  strata_clean <- gsub("\\*$", "", strata)
  strata_names <- names(strata)
  if (is.null(strata_names)) strata_names <- strata_clean
  strata_names <- ifelse(strata_names == "", strata_clean, strata_names)

  strata_def <- lapply(strata_clean, function(s) {
    parts <- strsplit(s, "\\|")[[1]]
    lapply(parts, function(p) as.integer(strsplit(p, "")[[1]]))
  })
  names(strata_def) <- strata_names

  list(
    strata_names = strata_names,
    strata_def   = strata_def,
    num_treatment = length(strata_def[[1]][[1]]),
    num_postrand  = length(strata_def[[1]])
  )
}

#' @noRd
parse_strata_list_of_vectors <- function(strata) {
  strata_names <- names(strata)
  if (is.null(strata_names) || any(strata_names == ""))
    stop("All strata must be named.")

  strata_def <- lapply(strata, function(v) list(as.integer(v)))
  names(strata_def) <- strata_names

  list(
    strata_names = strata_names,
    strata_def   = strata_def,
    num_treatment = length(strata[[1]]),
    num_postrand  = 1L
  )
}

#' @noRd
parse_strata_list_of_lists <- function(strata) {
  strata_names <- names(strata)
  if (is.null(strata_names) || any(strata_names == ""))
    stop("All strata must be named.")

  first_names <- names(strata[[1]])
  all_named   <- !is.null(first_names) && all(nzchar(first_names))
  all_unnamed <- is.null(first_names) || all(!nzchar(first_names))

  if (!all_named && !all_unnamed)
    stop("Mixed named and unnamed D variables not supported.")

  if (all_named) {
    for (i in seq_along(strata)) {
      if (!setequal(names(strata[[i]]), first_names))
        stop("All strata must have the same D variable names.")
    }
  }

  strata_def <- lapply(strata, function(s) lapply(s, as.integer))
  names(strata_def) <- strata_names

  list(
    strata_names = strata_names,
    strata_def   = strata_def,
    num_treatment = length(strata[[1]][[1]]),
    num_postrand  = length(strata[[1]])
  )
}

# ===========================================================================
# ER resolution
# ===========================================================================

#' @noRd
resolve_ER_model <- function(ER, strata_names) {
  n <- length(strata_names)
  if (is.null(ER)) return(rep(FALSE, n))
  if (is.logical(ER)) {
    stopifnot(length(ER) == n)
    return(ER)
  }
  if (is.character(ER)) {
    er_list <- rep(FALSE, n)
    for (name in ER) {
      idx <- which(strata_names == name)
      if (length(idx) == 0) warning("ER stratum '", name, "' not found, ignored.")
      else er_list[idx] <- TRUE
    }
    return(er_list)
  }
  stop("ER must be NULL, logical, or character vector.")
}

# ===========================================================================
# Family resolution
# ===========================================================================

#' @noRd
resolve_family_info <- function(family_name, link) {
  reg <- .family_registry[[family_name]]
  if (is.null(reg)) stop("Unsupported family: ", family_name)

  link_info <- reg$links[[link]]
  if (is.null(link_info)) stop("Unsupported link '", link, "' for family '", family_name, "'.")

  list(
    outcome_type = reg$outcome_type,
    stan_func    = reg$stan_func,
    stan_link    = link_info$stan_link,
    extra_params = reg$extra_params,
    equation     = link_info$eq
  )
}

# ===========================================================================
# Outcome group computation
# ===========================================================================

#' @noRd
compute_outcome_groups <- function(strata_names, strata_def, er_list, num_treatment) {
  labels <- character(0)

  for (i in seq_along(strata_names)) {
    if (er_list[i]) {
      labels <- c(labels, strata_names[i])
    } else {
      for (z in seq_len(num_treatment) - 1) {
        labels <- c(labels, paste0(strata_names[i], "(", z, ")"))
      }
    }
  }

  list(
    labels    = labels,
    n_groups  = length(labels),
    er_strata = strata_names[er_list]
  )
}

# ===========================================================================
# print
# ===========================================================================

#' @export
print.PStrataModel <- function(x, ...) {
  cat("PStrataModel\n")
  cat("  Stratum model:", deparse(x$stratum_formula), "\n")
  cat("  Outcome model:", deparse(x$outcome_formula), "\n")
  cat("  Family:", x$outcome_family$family, "(", x$outcome_family$link, ")\n")

  strata_strs <- vapply(seq_along(x$strata_names), function(i) {
    parts <- vapply(x$strata_def[[i]], function(d) paste(d, collapse = ""), character(1))
    paste0(x$strata_names[i], "(", paste(parts, collapse = "|"), ")")
  }, character(1))
  cat("  Strata:", paste(strata_strs, collapse = ", "), "\n")

  er_names <- x$strata_names[x$exclusion_restriction]
  if (length(er_names) > 0)
    cat("  Exclusion restriction:", paste(er_names, collapse = ", "), "\n")

  if (x$is_survival)
    cat("  Time points:", x$survival_time_points, "\n")

  invisible(x)
}

# ===========================================================================
# summary
# ===========================================================================

#' @export
summary.PStrataModel <- function(object, ...) {
  cat("PStrataModel Summary\n")
  cat("====================\n\n")

  # Call
  cat("Call:", deparse(object$call, width.cutoff = 200), "\n\n")

  # Variable roles
  cat("Treatment:", object$treatment_name, "\n")
  cat("Post-randomization:", paste(object$postrand_names, collapse = ", "), "\n")
  if (object$is_survival) {
    cat("Time-to-event outcome:", object$outcome_name, "\n")
    cat("Event indicator:", object$event_name, "(1 = observed, 0 = censored)\n")
  } else {
    cat("Outcome:", object$outcome_name, "\n")
  }

  # Strata table
  cat("\nStrata:\n")
  print_strata_table(object)

  # Stratum model
  cat("\nStratum model (softmax):\n")
  cat("  Stratum:", deparse(object$stratum_formula), "\n")
  if (length(object$s_terms$covariates) > 0)
    cat("  X = [", paste(object$s_terms$covariates, collapse = ", "), "]\n", sep = "")
  re_s <- if (length(object$s_random_effects) > 0) " + r_{s,G}" else ""
  s_eq <- build_linear_predictor("s", object$s_terms, re_s)
  cat("  P(s | X) = softmax(", s_eq, ") for s in {",
      paste(object$strata_names, collapse = ", "), "}\n", sep = "")
  cat("  Priors:\n")
  print_fixed_priors("s", object$s_terms, object$priors)
  print_random_effects(object$s_random_effects, "s")

  # Outcome model
  family_name <- object$outcome_family$family
  family_link <- object$outcome_family$link
  if (object$is_survival) {
    method <- sub("survival_", "", family_name)
    cat("\nOutcome model (survival, ", method, "):\n", sep = "")
  } else {
    cat("\nOutcome model (", family_name, ", ", family_link, " link):\n", sep = "")
  }
  cat("  Outcome:", deparse(object$outcome_formula), "\n")
  if (length(object$y_terms$covariates) > 0)
    cat("  X = [", paste(object$y_terms$covariates, collapse = ", "), "]\n", sep = "")

  re_g <- if (length(object$y_random_effects) > 0) " + r_{g,G}" else ""
  g_eq <- build_linear_predictor("g", object$y_terms, re_g)
  cat("  eta_g = ", g_eq, "\n", sep = "")

  groups_str <- paste(object$outcome_groups$labels, collapse = ", ")
  cat("  ", object$family_info$equation, " for g in {", groups_str, "}\n", sep = "")

  if (length(object$outcome_groups$er_strata) > 0) {
    cat("  Note:", paste(object$outcome_groups$er_strata, collapse = ", "),
        "shared across treatment arms (ER)\n")
  }

  if (object$is_survival)
    cat("  Time points:", object$survival_time_points, "(determined at fit time)\n")

  cat("  Priors:\n")
  print_fixed_priors("g", object$y_terms, object$priors)
  for (ep in object$family_info$extra_params) {
    cat("    ", paste0(ep$name, "_g ~ "), format_prior(object$priors[[ep$name]]), "\n", sep = "")
  }
  print_random_effects(object$y_random_effects, "g")

  # Available estimands
  cat("\nAvailable estimands after fitting:\n")
  if (object$is_survival) {
    cat("  survival probability, RACE (restricted average causal effect)\n")
  } else {
    cat("  mean outcome per stratum and treatment arm\n")
  }

  invisible(object)
}

# ===========================================================================
# Display helpers
# ===========================================================================

#' Build the linear predictor string based on what terms are present
#' @noRd
build_linear_predictor <- function(idx, terms, re_str) {
  parts <- character(0)
  if (terms$has_intercept) parts <- c(parts, paste0("a_", idx))
  if (length(terms$covariates) > 0) parts <- c(parts, paste0("b_", idx, " * X"))
  eq <- paste(parts, collapse = " + ")
  if (nzchar(re_str)) eq <- paste0(eq, re_str)
  eq
}

#' Print prior lines for fixed effects, respecting intercept/covariate presence
#' @noRd
print_fixed_priors <- function(idx, terms, priors) {
  if (terms$has_intercept)
    cat("    a_", idx, ": intercept ~ ", format_prior(priors$intercept), "\n", sep = "")
  if (length(terms$covariates) > 0)
    cat("    b_", idx, ": coefficients ~ ", format_prior(priors$coefficient), "\n", sep = "")
}

#' @noRd
format_prior <- function(prior) {
  if (prior$name == "flat") return("flat")
  params <- paste(unlist(prior$args), collapse = ", ")
  paste0(prior$name, "(", params, ")")
}

#' @noRd
print_strata_table <- function(object) {
  sd   <- object$strata_def
  sn   <- object$strata_names
  pn   <- object$postrand_names
  tn   <- object$treatment_name
  nt   <- object$num_treatment
  er   <- object$exclusion_restriction
  z_levels <- seq_len(nt) - 1

  # Column headers
  col_h <- character(0)
  for (d in pn) {
    for (z in z_levels) col_h <- c(col_h, paste0(d, "(", tn, "=", z, ")"))
  }
  col_h <- c(col_h, "ER")

  # Row data
  rows <- lapply(seq_along(sn), function(i) {
    vals <- unlist(sd[[i]])
    c(as.character(vals), if (er[i]) "*" else "")
  })

  # Column widths
  all_vals <- do.call(rbind, rows)
  cw <- pmax(nchar(col_h), apply(all_vals, 2, function(x) max(nchar(x))))
  nw <- max(nchar(sn))

  # Header
  hdr <- paste0(
    "  ", formatC("", width = nw), "  ",
    paste(mapply(function(h, w) formatC(h, width = w), col_h, cw), collapse = "  ")
  )
  cat(hdr, "\n")

  # Rows
  for (i in seq_along(sn)) {
    r <- paste0(
      "  ", formatC(sn[i], width = nw), "  ",
      paste(mapply(function(v, w) formatC(v, width = w), rows[[i]], cw), collapse = "  ")
    )
    cat(r, "\n")
  }
}

#' @noRd
print_random_effects <- function(re_list, idx_symbol) {
  if (is.null(re_list) || length(re_list) == 0) return(invisible())

  for (re in re_list) {
    cat("  Random effects ", re$formula_str, ":\n", sep = "")
    sym <- paste0("r_{", idx_symbol, ",", re$group, "}")
    if (!re$has_slopes) {
      cat("    ", sym, " ~ normal(0, tau), tau ~ inv_gamma(1, 1)\n", sep = "")
    } else {
      if (re$has_intercept) {
        cat("    ", sym, " has intercept and slope components\n", sep = "")
      } else {
        cat("    ", sym, " has slope components\n", sep = "")
      }
      cat("    each component ~ normal(0, tau), tau ~ inv_gamma(1, 1)\n")
    }
  }
}

#' Stan Code for PStrata Models
#'
#' Generate the Stan code corresponding to the model.
#' This is called internally by \code{\link{fit}}; use
#' \code{\link{stancode.PStrataFit}} to retrieve the code
#' from a fitted model.
#'
#' @param object An internal model specification object.
#' @param filename (optional) string. If not \code{NULL}, the
#'   Stan file is saved to this path.
#' @param debug only for development/testing use.
#'
#' @return A string containing the Stan program.
#'
#' @examples
#' \donttest{
#' data(sim_data_normal)
#' model <- PStrataModel(
#'   S.formula = Z + D ~ 1,
#'   Y.formula = Y ~ 1,
#'   Y.family  = gaussian(),
#'   strata    = c(n = "00", c = "01", a = "11"),
#'   ER        = c("n", "a")
#' )
#' ps_fit <- fit(model, data = sim_data_normal, chains = 2, iter = 500)
#' cat(ps_fit$stancode)
#' }
#'
#' @export
make_stancode <- function(object, filename = NULL, debug = FALSE) {
  PSobject <- if (inherits(object, "PStrata")) object$.design else object
  ctx <- build_stan_context(PSobject, debug)

  blocks <- c(
    stan_comment_lines(),
    " ",
    stan_functions_block(ctx),
    " ",
    stan_data_block(ctx),
    " ",
    stan_transformed_data_block(ctx),
    " ",
    stan_parameters_block(ctx),
    " ",
    stan_transformed_parameters_block(ctx),
    " ",
    stan_model_block(ctx),
    " ",
    stan_generated_quantities_block(ctx)
  )

  full_text <- paste0(blocks, collapse = "\n")

  if (!is.null(filename)) {
    if (!stringr::str_ends(filename, ".stan"))
      filename <- paste0(filename, ".stan")
    cat(full_text, file = filename)
  }

  full_text
}

# =============================================================================
# Context builder
# =============================================================================

#' Retrieve an inst/ config file as character lines
#'
#' Uses internal package data (R/sysdata.rda) when available.
#' Falls back to reading from inst/ for development/debug use.
#' @noRd
read_inst_file <- function(filename, debug) {
  if (!debug) {
    varname <- paste0(".", gsub("\\.txt$", "", filename), "_lines")
    val <- tryCatch(get(varname, envir = asNamespace("PStrata")),
                    error = function(e) NULL)
    if (!is.null(val)) return(val)
  }
  # Fallback: read from file (debug mode or sysdata not yet built)
  candidates <- if (debug) {
    c(file.path("inst", filename), file.path("PStrata", "inst", filename))
  } else {
    pkg_path <- path.package("PStrata")
    c(file.path(pkg_path, filename), file.path(pkg_path, "inst", filename))
  }
  for (path in candidates) {
    if (file.exists(path)) return(readLines(path))
  }
  stop("Cannot find config file: ", filename)
}

#' Parse family_info.txt to extract outcome type, Stan function, and extra params
#' @noRd
parse_family_info <- function(lines, Y_family) {
  Y_type <- NULL
  func_family <- NULL
  parameter_list <- list()

  for (line in lines) {
    parts <- stringr::str_split(line, " +")[[1]]
    if (parts[1] != Y_family) next

    Y_type <- parts[2]
    func_family <- parts[3]
    n_params <- as.integer(parts[4])
    if (n_params > 0) {
      for (i in seq_len(n_params)) {
        parameter_list[[i]] <- list(
          name = parts[2 * i + 3],
          type = parts[2 * i + 4]
        )
      }
    }
    break
  }

  list(Y_type = Y_type, func_family = func_family, parameter_list = parameter_list)
}

#' Parse link_info.txt to find the Stan inverse-link function name
#' @noRd
parse_link_info <- function(lines, Y_family, Y_link) {
  for (line in lines) {
    parts <- stringr::str_split(line, " +")[[1]]
    if (parts[1] == Y_family && parts[2] == Y_link) return(parts[3])
  }
  NULL
}

#' Build the prior list from PSobject for Stan code generation
#' @noRd
build_prior_list <- function(PSobject) {
  prior_names <- c("intercept", "coefficient", "sigma", "alpha", "lambda", "theta")
  lapply(prior_names, function(name) {
    prior_curr <- PSobject[[paste0("prior_", name)]]
    list(
      type  = name,
      func  = prior_curr$name,
      param = unlist(prior_curr$args)
    )
  })
}

#' Detect whether new Stan array syntax is needed (Stan >= 2.33)
#' @noRd
use_new_stan_array_syntax <- function() {
  tryCatch(
    utils::compareVersion(rstan::stan_version(), "2.33.0") >= 0,
    error = function(e) FALSE
  )
}

#' Generate a Stan array declaration in the correct syntax for the installed Stan version
#' @param new_syntax logical; TRUE for new syntax (array[dims] type name), FALSE for old (type name[dims])
#' @param indent character; leading whitespace
#' @param dims character; dimension expression (e.g. "N", "mx['G']")
#' @param base_type character; the type (e.g. "int<lower=0, upper=1>", "matrix[N, T]")
#' @param varname character; the variable name
#' @param comment character; optional trailing comment
#' @noRd
stan_array_decl <- function(new_syntax, indent, dims, base_type, varname, comment = "") {
  if (new_syntax) {
    decl <- paste0(indent, "array[", dims, "] ", base_type, " ", varname, ";")
  } else {
    decl <- paste0(indent, base_type, " ", varname, "[", dims, "];")
  }
  if (nzchar(comment)) paste0(decl, " ", comment) else decl
}

#' Assemble all derived quantities needed for Stan code generation
#' @noRd
build_stan_context <- function(PSobject, debug) {
  SZDG_max <- apply(PSobject$SZDG_table, 2, max)
  Y_family <- PSobject$Y.family$family
  Y_link   <- PSobject$Y.family$link

  family_info <- parse_family_info(
    read_inst_file("family_info.txt", debug), Y_family
  )
  func_link <- parse_link_info(
    read_inst_file("link_info.txt", debug), Y_family, Y_link
  )

  list(
    PSobject       = PSobject,
    SZDG_max       = SZDG_max,
    S_re           = length(PSobject$S.formula$random_eff_list),
    Y_re           = length(PSobject$Y.formula$random_eff_list),
    Y_family       = Y_family,
    Y_link         = Y_link,
    Y_type         = family_info$Y_type,
    func_family    = family_info$func_family,
    func_link      = func_link,
    parameter_list = family_info$parameter_list,
    prior_list     = build_prior_list(PSobject),
    func_imp_lines = read_inst_file("function_implement.txt", debug),
    new_syntax     = use_new_stan_array_syntax()
  )
}

# =============================================================================
# Random effect line generators (shared across multiple blocks)
# =============================================================================

#' Generate Stan data declarations for random effects
#' @param prefix "S" for stratum model, "G" for outcome model
#' @param re_count Number of random effect groups
#' @noRd
stan_re_data_lines <- function(prefix, re_count) {
  if (re_count == 0) return(character(0))
  model_desc <- if (prefix == "S") "principal stratum" else "outcome"
  lines <- paste0("    // random effect for ", model_desc, " model")
  for (i in seq_len(re_count)) {
    p <- paste0("P", prefix, "_RE_", i)
    n <- paste0("N", prefix, "_RE_", i)
    x <- paste0("X", prefix, "_RE_", i)
    lines <- c(lines,
      paste0("    int<lower=0> ", p, "; // number of random effect terms"),
      paste0("    int<lower=0> ", n, "; // number of levels"),
      paste0("    matrix[N, ", p, "*", n, "] ", x, "; // model matrix for random effect")
    )
  }
  lines
}

#' Generate Stan parameter declarations for random effects
#' @noRd
stan_re_param_lines <- function(prefix, max_dim, re_count, new_syntax) {
  if (re_count == 0) return(character(0))
  model_desc <- if (prefix == "S") "principal stratum" else "outcome"
  lines <- paste0("    // random effect for ", model_desc, " model")
  for (i in seq_len(re_count)) {
    lines <- c(lines,
      paste0("    matrix[", max_dim, ", P", prefix, "_RE_", i,
             "*N", prefix, "_RE_", i, "] beta_", prefix, "_RE_", i, ";"),
      stan_array_decl(new_syntax, "    ",
        paste0(max_dim, ", P", prefix, "_RE_", i),
        "real<lower=0>",
        paste0("tau_", prefix, "_RE_", i))
    )
  }
  lines
}

#' Generate transformed parameter declarations for random effects
#' @noRd
stan_re_transformed_decl <- function(prefix, max_dim, re_count, new_syntax) {
  if (re_count == 0) return(character(0))
  lines <- character(0)
  for (i in seq_len(re_count)) {
    lines <- c(lines,
      stan_array_decl(new_syntax, "    ", max_dim,
        paste0("matrix[N", prefix, "_RE_", i, ", P", prefix, "_RE_", i, "]"),
        paste0("M_beta_", prefix, "_RE_", i))
    )
  }
  lines
}

#' Generate transformed parameter body (reshape) for random effects
#' @param prefix "S" or "G"
#' @param re_count Number of RE groups
#' @param loop_bound The Stan for-loop upper bound
#' @param body_count Number of assignment lines to generate inside the loop
#' @noRd
stan_re_transformed_body <- function(prefix, re_count, loop_bound, body_count) {
  if (re_count == 0) return(character(0))
  lines <- character(0)
  for (i in seq_len(re_count)) {
    lines <- c(lines, paste0("    for (i in 1:", loop_bound, "){"))
    for (j in seq_len(body_count)) {
      lines <- c(lines,
        paste0("        M_beta_", prefix, "_RE_", i, "[", j,
               "] = to_matrix(beta_", prefix, "_RE_", i, "[", j,
               "], N", prefix, "_RE_", i, ", P", prefix, "_RE_", i, ", 0);")
      )
    }
    lines <- c(lines, "    }")
  }
  lines
}

#' Generate model-block priors for random effects
#' @noRd
stan_re_prior_lines <- function(prefix, max_dim, re_count) {
  if (re_count == 0) return(character(0))
  lines <- character(0)
  for (i in seq_len(re_count)) {
    lines <- c(lines,
      paste0("    for (i in 1:", max_dim, ") {"),
      paste0("        tau_", prefix, "_RE_", i, "[i] ~ inv_gamma(1, 1);"),
      paste0("        for (j in 1:P", prefix, "_RE_", i, ") {"),
      paste0("            M_beta_", prefix, "_RE_", i,
             "[i][:, j] ~ normal(0, tau_", prefix, "_RE_", i, "[i, j]);"),
      "        }",
      "    }"
    )
  }
  lines
}

#' Generate the RE linear predictor addition terms (e.g. " + XS_RE_1[n] * beta_S_RE_1[s-1]'")
#' @noRd
stan_re_linear_terms <- function(prefix, re_count, row_idx, coef_idx) {
  if (re_count == 0) return("")
  paste(vapply(seq_len(re_count), function(i) {
    paste0(" + X", prefix, "_RE_", i, "[", row_idx, "] * beta_", prefix, "_RE_", i, "[", coef_idx, "]'")
  }, character(1)), collapse = "")
}

#' Generate RE matrix linear terms (no row index, for generated quantities)
#' @noRd
stan_re_matrix_terms <- function(prefix, re_count) {
  if (re_count == 0) return("")
  paste(vapply(seq_len(re_count), function(i) {
    paste0(" + X", prefix, "_RE_", i, " * beta_", prefix, "_RE_", i, "'")
  }, character(1)), collapse = "")
}

# =============================================================================
# Stan block generators
# =============================================================================

#' @noRd
stan_comment_lines <- function() {
  c(
    "// Automatically generated by PStrata 0.0.4",
    paste0("// at ", Sys.time(), " ", Sys.timezone())
  )
}

#' @noRd
stan_functions_block <- function(ctx) {
  lines <- character(0)
  aim <- FALSE
  for (line in ctx$func_imp_lines) {
    if (line == "<<<>>>") aim <- FALSE
    if (!aim) {
      parts <- stringr::str_split(line, " ")[[1]]
      if (parts[1] == "<<<") {
        if (parts[2] %in% c(ctx$func_family, ctx$func_link))
          aim <- TRUE
      }
      next
    }
    lines <- c(lines, paste0("    ", line))
  }
  if (length(lines) > 0) c("functions {", lines, "}") else character(0)
}

#' @noRd
stan_data_block <- function(ctx) {
  mx <- ctx$SZDG_max
  ns <- ctx$new_syntax
  lines <- c(
    "data {",
    "    int<lower=0> N; // number of observations",
    "    int<lower=0> PS; // number of predictors for principal stratum model",
    "    int<lower=0> PG; // number of predictors for outcome model",
    stan_array_decl(ns, "    ", "N", paste0("int<lower=0, upper=", mx[2], ">"), "Z", "// treatment arm"),
    stan_array_decl(ns, "    ", "N", paste0("int<lower=0, upper=", mx[3], ">"), "D", "// post randomization confounding variable")
  )

  Y_lines <- switch(ctx$Y_type,
    "real"     = stan_array_decl(ns, "    ", "N", "real", "Y", "// real outcome"),
    "positive" = stan_array_decl(ns, "    ", "N", "real<lower=0>", "Y", "// positive outcome"),
    "binary"   = stan_array_decl(ns, "    ", "N", "int<lower=0, upper=1>", "Y", "// binary outcome"),
    "count"    = stan_array_decl(ns, "    ", "N", "int<lower=0>", "Y", "// count outcome"),
    "survival" = c(
      stan_array_decl(ns, "    ", "N", "real<lower=0>", "Y", "// survival outcome"),
      stan_array_decl(ns, "    ", "N", "int<lower=0, upper=1>", "delta", "// event indicator"),
      "    int<lower=0> T; // number of time points for evaluation",
      stan_array_decl(ns, "    ", "T", "real<lower=0>", "time", "// time points")
    )
  )

  c(lines, Y_lines,
    "    matrix[N, PS] XS; // model matrix for principal stratum model",
    "    matrix[N, PG] XG; // model matrix for outcome model",
    stan_re_data_lines("S", ctx$S_re),
    stan_re_data_lines("G", ctx$Y_re),
    "}"
  )
}

#' @noRd
stan_transformed_data_block <- function(ctx) {
  mx <- ctx$SZDG_max
  SG_table <- unique(ctx$PSobject$SZDG_table[, c("S", "G")])

  lines <- c(
    "transformed data {",
    stan_array_decl(ctx$new_syntax, "    ", mx[4], "int", "S")
  )
  for (i in seq_len(nrow(SG_table))) {
    lines <- c(lines,
      paste0("   S[", SG_table[i, "G"], "] = ", SG_table[i, "S"] + 1, ";")
    )
  }
  c(lines, "}")
}

#' @noRd
stan_parameters_block <- function(ctx) {
  mx <- ctx$SZDG_max
  ns <- ctx$new_syntax
  lines <- c(
    "parameters {",
    paste0("    matrix[", mx['S'], ", PS] beta_S; // coefficients for principal stratum model"),
    paste0("    matrix[", mx['G'], ", PG] beta_G; // coefficients for outcome model"),
    stan_re_param_lines("S", mx['S'], ctx$S_re, ns),
    stan_re_param_lines("G", mx['G'], ctx$Y_re, ns)
  )

  for (param in ctx$parameter_list) {
    base_type <- if (param$type == "positive") "real<lower=0>" else "real"
    lines <- c(lines,
      stan_array_decl(ns, "    ", mx['G'], base_type, param$name)
    )
  }

  c(lines, "}")
}

#' @noRd
stan_transformed_parameters_block <- function(ctx) {
  mx <- ctx$SZDG_max
  c(
    "transformed parameters {",
    stan_re_transformed_decl("S", mx['S'], ctx$S_re, ctx$new_syntax),
    stan_re_transformed_decl("G", mx['G'], ctx$Y_re, ctx$new_syntax),
    stan_re_transformed_body("S", ctx$S_re, mx['S'], mx['S']),
    stan_re_transformed_body("G", ctx$Y_re, mx['S'], mx['G']),
    "}"
  )
}

#' @noRd
stan_model_block <- function(ctx) {
  mx <- ctx$SZDG_max
  PSobject <- ctx$PSobject

  lines <- c(
    "model {",
    "    // random effect",
    stan_re_prior_lines("S", mx['S'], ctx$S_re),
    stan_re_prior_lines("G", mx['G'], ctx$Y_re),
    "    // prior",
    stan_prior_lines(ctx),
    "    // model",
    stan_likelihood_lines(ctx)
  )

  c(lines, "}")
}

#' Generate prior statements for the model block
#' @noRd
stan_prior_lines <- function(ctx) {
  lines <- character(0)
  for (prior in ctx$prior_list) {
    if (prior$func == "flat") next
    params_str <- paste(prior$param, collapse = ", ")

    if (prior$type == "intercept") {
      lines <- c(lines,
        paste0("    beta_S[:, 1] ~ ", prior$func, "(", params_str, ");"),
        paste0("    beta_G[:, 1] ~ ", prior$func, "(", params_str, ");")
      )
    } else if (prior$type == "coefficient") {
      lines <- c(lines,
        "    if (PS >= 2)",
        paste0("        to_vector(beta_S[:, 2:PS]) ~ ", prior$func, "(", params_str, ");"),
        "    if (PG >= 2)",
        paste0("        to_vector(beta_G[:, 2:PG]) ~ ", prior$func, "(", params_str, ");")
      )
    } else {
      is_model_param <- any(vapply(ctx$parameter_list,
        function(p) p$name == prior$type, logical(1)))
      if (is_model_param) {
        lines <- c(lines,
          paste0("    ", prior$type, " ~ ", prior$func, "(", params_str, ");")
        )
      }
    }
  }
  lines
}

#' Generate the main likelihood loop for the model block
#' @noRd
stan_likelihood_lines <- function(ctx) {
  mx <- ctx$SZDG_max
  PSobject <- ctx$PSobject
  S_re_term <- stan_re_linear_terms("S", ctx$S_re, "n", "s-1")

  lines <- c(
    "    for (n in 1:N) {",
    "        int length;",
    stan_array_decl(ctx$new_syntax, "        ", mx['S'] + 1, "real", "log_prob"),
    "        log_prob[1] = 0;",
    paste0("        for (s in 2:", mx['S'] + 1, ") {"),
    paste0("            log_prob[s] = XS[n] * beta_S[s-1]'", S_re_term, ";"),
    "        }"
  )

  ZD_comb <- unique(PSobject$SZDG_table[, c("Z", "D")])

  # length assignment block
  b_else <- FALSE
  for (r in seq_len(nrow(ZD_comb))) {
    z <- ZD_comb[r, "Z"]; d <- ZD_comb[r, "D"]
    n_groups <- sum(PSobject$SZDG_table[, "Z"] == z & PSobject$SZDG_table[, "D"] == d)
    lines <- c(lines,
      paste0("        ", if (b_else) "else ", "if (Z[n] == ", z, " && D[n] == ", d, ")"),
      paste0("            length = ", n_groups, ";")
    )
    b_else <- TRUE
  }
  # Guard: unrecognized (Z, D) combination — skip this observation
  lines <- c(lines, "        else length = 0;")

  lines <- c(lines,
    "        if (length > 0) {",
    stan_array_decl(ctx$new_syntax, "            ", "length", "real", "log_l")
  )

  # likelihood computation block
  b_else <- FALSE
  for (r in seq_len(nrow(ZD_comb))) {
    z <- ZD_comb[r, "Z"]; d <- ZD_comb[r, "D"]
    SG_table <- PSobject$SZDG_table[
      PSobject$SZDG_table[, "Z"] == z & PSobject$SZDG_table[, "D"] == d, , drop = FALSE
    ]
    lines <- c(lines,
      paste0("            ", if (b_else) "else ", "if (Z[n] == ", z, " && D[n] == ", d, ") {"),
      paste0("                // Z:", z, " D:", d, " S:", paste(SG_table[, "S"], collapse = "/"))
    )

    for (sg in seq_len(nrow(SG_table))) {
      s <- SG_table[sg, "S"]
      g <- SG_table[sg, "G"]
      Y_re_term <- stan_re_linear_terms("G", ctx$Y_re, "n", g)

      str_param <- ""
      if (length(ctx$parameter_list) > 0) {
        param_refs <- paste(vapply(ctx$parameter_list,
          function(p) paste0(p$name, "[", g, "]"), character(1)), collapse = ", ")
        str_param <- paste0(", ", param_refs,
          if (ctx$Y_type == "survival") ", delta[n]" else "")
      }

      lines <- c(lines,
        paste0("                log_l[", sg, "] = log_prob[", s + 1, "] + ",
               ctx$func_family, "(Y[n] | ",
               ctx$func_link, "(XG[n] * beta_G[", g, "]'", Y_re_term, ")",
               str_param, ");")
      )
    }
    lines <- c(lines, "            }")
    b_else <- TRUE
  }

  c(lines,
    "            target += log_sum_exp(log_l) - log_sum_exp(log_prob);",
    "        }",
    "    }"
  )
}

# =============================================================================
# Generated quantities block
# =============================================================================

#' @noRd
stan_generated_quantities_block <- function(ctx) {
  mx <- ctx$SZDG_max

  if (ctx$Y_type == "survival") {
    stan_gq_survival(ctx, mx)
  } else {
    stan_gq_ordinary(ctx, mx)
  }
}

#' Generated quantities for non-survival outcomes
#' @noRd
stan_gq_ordinary <- function(ctx, mx) {
  S_re_mat <- stan_re_matrix_terms("S", ctx$S_re)
  Y_re_term_fn <- function(idx_var) stan_re_linear_terms("G", ctx$Y_re, "i", idx_var)

  lines <- c(
    "generated quantities {",
    paste0("    vector[", mx['S'] + 1, "] strata_prob; // the probability of being in each stratum"),
    paste0("    vector[", mx['G'], "] mean_effect; // mean response"),
    "    {",
    paste0("        matrix[N, ", mx['S'] + 1, "] log_prob;"),
    paste0("        vector[", mx['G'], "] numer;"),
    paste0("        matrix[N, ", mx['G'], "] expected_mean;"),
    "        for (i in 1:N)",
    paste0("            for (j in 1:", mx['G'], ")"),
    paste0("                expected_mean[i, j] = ", ctx$func_link,
           "(XG[i] * beta_G[j]'", Y_re_term_fn("j"), ");"),
    "        log_prob[:, 1] = rep_vector(0, N);",
    paste0("        log_prob[:, 2:", mx['S'] + 1, "] = XS * beta_S'", S_re_mat, ";"),
    "        for (n in 1:N) {",
    "            log_prob[n] -= log_sum_exp(log_prob[n]);",
    "        }",
    paste0("        for (s in 1:", mx['S'] + 1, ") strata_prob[s] = mean(exp(log_prob[:, s]));"),
    paste0("        for (g in 1:", mx['G'], ") {"),
    "            numer[g] = mean(expected_mean[:, g] .* exp(log_prob[:, S[g]]));",
    "            mean_effect[g] = numer[g] / strata_prob[S[g]];",
    "        }",
    "    }",
    "}"
  )
  lines
}

#' Generated quantities for survival outcomes
#' @noRd
stan_gq_survival <- function(ctx, mx) {
  S_re_mat <- stan_re_matrix_terms("S", ctx$S_re)
  Y_re_term_fn <- function(idx_var) stan_re_linear_terms("G", ctx$Y_re, "i", idx_var)

  lines <- c(
    "generated quantities {",
    paste0("    vector[", mx['S'] + 1, "] strata_prob; // the probability of being in each stratum"),
    paste0("    matrix[", mx['G'], ", T] mean_surv_prob; // mean survival probability"),
    paste0("    matrix[", mx['G'], ", T] mean_RACE; // mean restricted average causal effect"),
    "    {",
    paste0("        matrix[N, ", mx['S'] + 1, "] log_prob;"),
    paste0("        matrix[", mx['G'], ", T] numer_surv_prob;"),
    paste0("        matrix[", mx['G'], ", T] numer_RACE;"),
    stan_array_decl(ctx$new_syntax, "        ", mx['G'], "matrix[N, T]", "expected_surv_prob"),
    stan_array_decl(ctx$new_syntax, "        ", mx['G'], "matrix[N, T]", "expected_RACE"),
    "        for (i in 1:N)",
    paste0("            for (j in 1:", mx['G'], ")"),
    "                for (t in 1:T) {",
    paste0("                    real mu = ", ctx$func_link,
           "(XG[i] * beta_G[j]'", Y_re_term_fn("j"), ");")
  )

  if (ctx$Y_family == "survival_Cox") {
    lines <- c(lines,
      "                    expected_surv_prob[j][i,t] = exp(-exp(mu) * pow(time[t], exp(theta[j])));",
      "                    expected_RACE[j][i,t] = exp(-theta[j] - mu * exp(-theta[j])) * tgamma(exp(-theta[j])) * gamma_p(exp(-theta[j]), exp(mu) * pow(time[t], exp(theta[j])));"
    )
  } else if (ctx$Y_family == "survival_AFT") {
    lines <- c(lines,
      "                    expected_surv_prob[j][i,t] = 1 - normal_cdf(log(time[t]), mu, sigma[j]);",
      "                    expected_RACE[j][i,t] = time[t] - normal_cdf(log(time[t]), mu, sigma[j]) + exp(mu + sigma[j] * sigma[j] / 2) * normal_cdf(log(time[t]), mu + sigma[j] * sigma[j], sigma[j]);"
    )
  }

  c(lines,
    "                }",
    "        log_prob[:, 1] = rep_vector(0, N);",
    paste0("        log_prob[:, 2:", mx['S'] + 1, "] = XS * beta_S'", S_re_mat, ";"),
    "        for (n in 1:N) {",
    "            log_prob[n] -= log_sum_exp(log_prob[n]);",
    "        }",
    paste0("        for (s in 1:", mx['S'] + 1, ") strata_prob[s] = mean(exp(log_prob[:, s]));"),
    paste0("        for (g in 1:", mx['G'], ") {"),
    "            for (t in 1:T) {",
    "                numer_surv_prob[g, t] = mean(expected_surv_prob[g][:, t] .* exp(log_prob[:, S[g]]));",
    "                numer_RACE[g, t] = mean(expected_RACE[g][:, t] .* exp(log_prob[:, S[g]]));",
    "                mean_surv_prob[g, t] = numer_surv_prob[g, t] / strata_prob[S[g]];",
    "                mean_RACE[g, t] = numer_RACE[g, t] / strata_prob[S[g]];",
    "            }",
    "        }",
    "    }",
    "}"
  )
}

# Internal utility functions for PStrata
# These are not exported and are used by multiple files.

# --- Array helpers ---

#' Subset an array along a specific dimension
#' @param arr An array
#' @param dim_idx Integer, the dimension to subset along
#' @param idx Index (numeric, logical, or character) to select
#' @return A subsetted array with drop = FALSE
#' @noRd
slice_array <- function(arr, dim_idx, idx) {
  indices <- rep(list(TRUE), length(dim(arr)))
  indices[[dim_idx]] <- idx
  do.call(`[`, c(list(arr), indices, list(drop = FALSE)))
}

#' Assign values to a slice of an array along a specific dimension
#' @noRd
assign_slice <- function(arr, dim_idx, idx, value) {
  indices <- rep(list(TRUE), length(dim(arr)))
  indices[[dim_idx]] <- idx
  do.call(`[<-`, c(list(arr), indices, list(value = value)))
}

# --- Contrast computation ---

#' Compute pairs of indices for contrast computation
#' @param levels Integer vector of dimension indices to contrast
#' @param type One of "all", "sequential", "cycle"
#' @return A data.frame with columns Var1, Var2 (each row is a pair to subtract)
#' @noRd
compute_contrast_pairs <- function(levels, type = c("all", "sequential", "cycle")) {
  type <- match.arg(type)
  if (type == "all") {
    pairs <- expand.grid(Var1 = levels, Var2 = levels)
    pairs[pairs[, 1] > pairs[, 2], , drop = FALSE]
  } else if (type == "sequential" || (type == "cycle" && length(levels) == 2)) {
    data.frame(Var1 = levels[-1], Var2 = levels[-length(levels)])
  } else { # cycle with > 2 levels
    data.frame(Var1 = c(levels[-1], levels[1]), Var2 = levels)
  }
}

#' Compute contrasts along a single dimension of an array
#'
#' Replaces the repetitive contrast logic in PSContrast for each dimension
#' and for survival/non-survival cases.
#'
#' @param arr An array (3D for non-survival, 4D for survival)
#' @param dim_idx Which dimension to take contrasts along (1=S, 2=Z, 3=T)
#' @param selection TRUE (all levels) or a logical/numeric index vector
#' @param type One of "all", "sequential", "cycle"
#' @return A new array with contrasts computed along the specified dimension
#' @noRd
contrast_along_dim <- function(arr, dim_idx, selection, type) {
  dim_arr <- dim(arr)
  name_arr <- dimnames(arr)

  if (is.logical(selection) && length(selection) == 1 && selection) {
    levels <- seq_len(dim_arr[dim_idx])
  } else {
    levels <- (seq_len(dim_arr[dim_idx]))[selection]
  }

  pairs <- compute_contrast_pairs(levels, type)

  new_dim <- dim_arr
  new_dim[dim_idx] <- nrow(pairs)

  new_names <- name_arr
  new_names[[dim_idx]] <- apply(pairs, 1, function(x) {
    paste0("{", name_arr[[dim_idx]][x[1]], "}-{", name_arr[[dim_idx]][x[2]], "}")
  })

  new_arr <- array(NA, dim = new_dim, dimnames = new_names)

  for (i in seq_len(nrow(pairs))) {
    diff <- slice_array(arr, dim_idx, pairs[i, 1]) -
            slice_array(arr, dim_idx, pairs[i, 2])
    new_arr <- assign_slice(new_arr, dim_idx, i, diff)
  }

  new_arr
}

# --- Summary helpers ---

#' Compute summary statistics (mean, sd, quantiles) over the last dimension
#'
#' The last dimension of the array is treated as the iteration/sample dimension.
#' Returns an array where the last dimension is the 7 summary statistics.
#'
#' @param arr An array where the last dimension is iterations
#' @return An array with the last dimension replaced by 7 summary stats
#' @noRd
summarize_last_dim <- function(arr) {
  ndim <- length(dim(arr))
  margin <- rev(seq_len(ndim - 1))
  summary_raw <- apply(arr, margin, function(x) {
    c(mean(x), stats::sd(x),
      stats::quantile(x, c(0.025, 0.25, 0.5, 0.75, 0.975)))
  })
  aperm(summary_raw, rev(seq_len(length(dim(summary_raw)))))
}

#' Standard summary statistic names
#' @noRd
summary_stat_names <- function() {
  c("mean", "sd", "2.5%", "25%", "median", "75%", "97.5%")
}

#' Convert a summary array to a matrix with labeled rows
#'
#' Flattens all dimensions except the last (statistics) into rows.
#' Row labels are colon-separated combinations of dimension names.
#' The ordering matches the original: first dim varies slowest, last data dim fastest.
#'
#' @param summary_array Array where last dimension is summary statistics
#' @return A matrix with rows labeled "dim1:dim2:..." and columns as stat names
#' @noRd
summary_array_to_matrix <- function(summary_array) {
  dims <- dim(summary_array)
  ndim <- length(dims)
  n_stats <- dims[ndim]
  data_dims <- dims[-ndim]
  n_rows <- prod(data_dims)

  mat <- matrix(NA, nrow = n_rows, ncol = n_stats)

  # Build index grid: last data dim varies fastest (innermost loop)
  idx_grid <- do.call(expand.grid, rev(lapply(data_dims, seq_len)))
  idx_grid <- idx_grid[, rev(seq_len(ncol(idx_grid))), drop = FALSE]

  for (i in seq_len(n_rows)) {
    idx <- c(as.list(idx_grid[i, ]), list(TRUE))
    mat[i, ] <- do.call(`[`, c(list(summary_array), idx, list(drop = TRUE)))
  }

  # Row names from dimension name combinations
  dim_names <- dimnames(summary_array)
  name_grid <- do.call(expand.grid, rev(lapply(dim_names[-ndim], identity)))
  name_grid <- name_grid[, rev(seq_len(ncol(name_grid))), drop = FALSE]
  rownames(mat) <- apply(name_grid, 1, paste, collapse = ":")
  colnames(mat) <- dim_names[[ndim]]

  mat
}

#' Convert a summary matrix to a data.frame with dimension columns
#'
#' Splits row names by ":" and prepends them as named columns.
#'
#' @param mat A summary matrix from summary_array_to_matrix
#' @param col_names Character vector of column names for the dimension columns
#' @param time_points Optional numeric vector; if provided, integer T values are
#'   converted to the corresponding time point values
#' @return A data.frame with dimension columns prepended
#' @noRd
summary_matrix_to_df <- function(mat, col_names, time_points = NULL) {
  df <- as.data.frame(mat)
  parts <- purrr::transpose(stringr::str_split(rownames(df), ":"))

  prefix_df <- as.data.frame(
    stats::setNames(
      lapply(seq_along(col_names), function(i) unlist(parts[[i]])),
      col_names
    ),
    stringsAsFactors = FALSE
  )

  # Convert T column to numeric time points if applicable
  if ("T" %in% col_names && !is.null(time_points)) {
    t_col <- prefix_df$T
    t_int <- suppressWarnings(as.integer(t_col))
    if (!any(is.na(t_int))) {
      prefix_df$T <- time_points[t_int]
    }
  }

  cbind(prefix_df, df)
}

#' Simulated Dataset for Normal Outcome
#'
#' A dataset generated for illustration of the principal stratification analysis.
#' This dataset represents the common case of non-compliance.
#' 
#' @details 
#' The dataset represents the scenario where actual treatment might not be in compliance
#' with the randomized (assigned) treatment. Defiers are ruled out, leaving three
#' strata, "never taker", "complier" and "always taker" randomly sampled with 
#' probability 0.3, 0.2 and 0.5 respectively. The assigned treatment \eqn{Z} is randomized
#' with 0.5 probability for either arm. The outcome \eqn{Y} is given by the following.
#' \describe{
#' \item{never taker}{\eqn{Y \sim N(3, 1)}}
#' \item{complier}{\eqn{Y \sim N(-1-Z, 0.5)}}
#' \item{always taker}{\eqn{Y \sim N(1, 2)}}
#' }
#' The exclusion restriction assumption holds for never takers and always takers in this
#' generated dataset.
#'
#' @format ## `sim_data_normal`
#' A data frame with 1,000 rows and 4 columns:
#' \describe{
#'   \item{S}{Principal Strata: "never taker", "complier" or "always taker"}
#'   \item{Z}{Randomized treatment arm: 0 = control, 1 = treatment}
#'   \item{D}{Actual treatment arm: 0 = control, 1 = treatment}
#'   \item{Y}{Outcome}
#' }
#' @examples
#' data(sim_data_normal)
#' head(sim_data_normal)
#' table(sim_data_normal$D, sim_data_normal$Z)
"sim_data_normal"

#' PStrata: Principal STRATification Analysis for Data with Post-Randomization Confounding
#'
#' The \bold{PStrata} package is designed for estimating causal effects in the presence of post-treatment confounding using principal stratification.
#' It provides an interface to fit the Bayesian principal stratification model, which is a complex mixture model, using \bold{Stan}, a C++ package for obtaining full Bayesian inference.
#' The formula syntax is an extended version of the syntax applied in many regression functions and packages, such as \code{\link{lm}}, \code{\link{glm}} and \pkg{\link[lme4]{lme4-package}}, to provide a simple interface.
#' A wide variety of distributions and link functions are supported, allowing users to fit linear, binary or count data, and survival models with principal stratification.
#' Further modeling options include multiple post-treatment confounding variables and cluster random effects.
#' The monotonicity and exclusion restriction assumptions can be easily applied, and
#' prior specifications are flexible and encourage users to reflect their prior belief.
#' In addition, all parameters can be inferred from the posterior distribution, which enables further analysis other than provided by the package.
#'
#' @name PStrata_PACKAGE
#' @details
#' The Bayesian principal stratification analysis relies on two models, the principal stratum model and the outcome model.
#' The main workflow is:
#' \enumerate{
#'   \item \code{\link{PStrataModel}}: specify the model (formulas, strata, priors).
#'   \item \code{\link{fit}}: compile and run MCMC sampling via Stan.
#'   \item \code{\link{estimate}}: extract posterior potential outcomes.
#'   \item \code{\link{contrast}}: compute causal effect contrasts.
#' }
#' Based on the supplied formulas, data and additional information,
#' it automatically generates the Stan code via \code{\link{make_stancode}} and fits the model using \pkg{Stan}.
#'
#' The estimated probability for each principal stratum and the estimated mean response are calculated with \pkg{Stan} as it is faster and more space-efficient.
#' Post-processing methods \code{\link{summary}} and \code{\link{plot}} provide overviews and visualization.
#'
#' Because \pkg{PStrata} heavily relies on \pkg{Stan} for posterior sampling, a C++ compiler is required.
#' The program \pkg{Rtools} (available on \url{https://cran.r-project.org/bin/windows/Rtools/}) comes with a C++ compiler for Windows.
#' On Mac, Xcode is suggested. For further instructions on how to get the compilers running, please refer to the prerequisites section at the \href{https://github.com/stan-dev/rstan/wiki/RStan-Getting-Started}{RStan-Getting-Started} page.
#'
#' @references
#' The Stan Development Team. Stan Modeling Language User's Guide and Reference Manual. \url{https://mc-stan.org/users/documentation/}
#'
#' Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. \url{https://mc-stan.org/}
#' @seealso \code{\link{PStrataModel}}, \code{\link{fit}}, \code{\link{estimate}}, \code{\link{contrast}}
#' @importFrom stats coef formula nobs
NULL

# Suppress R CMD check NOTEs for ggplot2 aes() variables
utils::globalVariables(c("value", "Z", "S", "T", "mean", "label",
                         "2.5%", "97.5%"))

#' Prior Distribution Functions
#'
#' Define prior functions used in \code{PStrata}.
#'
#' @param mu,sigma,df,alpha,beta parameters for the prior distribution
#'
#' @return A list, including the following items.
#' \describe{
#' \item{name}{name of the distribution}
#' \item{type}{type of the distribution, one character string of "real" or "positive"}
#' \item{args}{a named list of all the input parameters}
#' \item{call}{a function call object of the prior distribution on the parameters}
#' }
#'
#' @examples
#' prior_flat()
#' prior_normal(0, 10)
#' prior_t(0, 2.5, df = 3)
#' prior_cauchy(0, 5)
#' prior_inv_gamma(2, 1)
#'
#' @name prior
#' @rdname prior
NULL

#' Construct a prior distribution object (internal factory)
#' @param name Stan distribution name
#' @param type "real" or "positive"
#' @param ... Named arguments in Stan argument order
#' @return A prior list
#' @noRd
make_prior <- function(name, type, ...) {
  args <- list(...)
  call_obj <- if (name == "flat") {
    NULL
  } else {
    as.call(c(list(as.name(name)), unname(args)))
  }
  list(name = name, type = type, args = args, call = call_obj)
}

#' @rdname prior
#' @export
prior_flat <- function() {
  make_prior("flat", "real")
}

#' @rdname prior
#' @export
prior_normal <- function(mu = 0, sigma = 1) {
  make_prior("normal", "real", mu = mu, sigma = sigma)
}

#' @rdname prior
#' @export
prior_t <- function(mu = 0, sigma = 1, df = 1) {
  make_prior("t", "real", df = df, mu = mu, sigma = sigma)
}

#' @rdname prior
#' @export
prior_cauchy <- function(mu = 0, sigma = 1) {
  make_prior("cauchy", "real", mu = mu, sigma = sigma)
}

#' @rdname prior
#' @export
prior_lasso <- function(mu = 0, sigma = 1) {
  make_prior("double_exponential", "real", mu = mu, sigma = sigma)
}

#' @rdname prior
#' @export
prior_logistic <- function(mu = 0, sigma = 1) {
  make_prior("logistic", "real", mu = mu, sigma = sigma)
}

#' @rdname prior
#' @export
prior_chisq <- function(df = 1) {
  make_prior("chi_square", "positive", df = df)
}

#' @rdname prior
#' @export
prior_inv_chisq <- function(df = 1) {
  make_prior("inv_chi_square", "positive", df = df)
}

#' @rdname prior
#' @export
prior_exponential <- function(beta = 1) {
  make_prior("exponential", "positive", beta = beta)
}

#' @rdname prior
#' @export
prior_gamma <- function(alpha = 1, beta = 1) {
  make_prior("gamma", "positive", alpha = alpha, beta = beta)
}

#' @rdname prior
#' @export
prior_inv_gamma <- function(alpha = 1, beta = 1) {
  make_prior("inv_gamma", "positive", alpha = alpha, beta = beta)
}

#' @rdname prior
#' @export
prior_weibull <- function(alpha = 1, sigma = 1) {
  make_prior("weibull", "positive", alpha = alpha, sigma = sigma)
}

# ===========================================================================
# fit: Estimate a PStrataModel with data via Stan MCMC
# ===========================================================================

#' Fit a Principal Stratification Model
#'
#' Estimate a principal stratification model by running MCMC via Stan.
#' The model specification (a \code{\link{PStrataModel}}) is combined with
#' observed data to produce posterior samples.
#'
#' @param model A \code{\link{PStrataModel}} object.
#' @param data A data frame containing all variables referenced in the model.
#' @param chains,iter,warmup,cores,seed MCMC settings passed to
#'   \code{\link[rstan]{stan}}.
#' @param .debug Logical. If TRUE, read Stan helper files from \code{inst/}
#'   rather than installed package data. Useful during development.
#' @param ... Additional arguments passed to \code{\link[rstan]{stan}}.
#'
#' @return An object of class \code{PStrataFit}
#'   (or \code{PStrataFitSurvival}).
#' @examples
#' \donttest{
#' data(sim_data_normal)
#' model <- PStrataModel(
#'   S.formula = Z + D ~ 1,
#'   Y.formula = Y ~ 1,
#'   Y.family  = gaussian(),
#'   strata    = c(n = "00", c = "01", a = "11"),
#'   ER        = c("n", "a")
#' )
#' ps_fit <- fit(model, data = sim_data_normal, chains = 2, iter = 500)
#' summary(ps_fit)
#' diagnostics(ps_fit)
#' plot(ps_fit)
#' cat(ps_fit$stancode)
#' }
#' @export
fit <- function(model, ...) UseMethod("fit")

#' @rdname fit
#' @export
fit.PStrataModel <- function(model, data, chains = 4, iter = 2000,
                              warmup = floor(iter / 2), cores = 1,
                              seed = NULL, .debug = FALSE, ...) {
  cl <- match.call()
  stopifnot(is.data.frame(data))

  # 1. Build design matrices and extract columns from data
  dm <- build_design_matrices(model, data)

  # 2. Encode strata definition as integer matrix
  strata_enc <- encode_strata(model$strata_def, model$num_treatment)

  # 3. Build SZDG mapping table
  SZDG <- build_SZDG(
    strata_enc$strata_matrix, model$exclusion_restriction,
    model$num_strata, model$num_treatment
  )

  # 4. Assemble Stan data list
  standata <- assemble_standata(model, dm, strata_enc, SZDG)

  # 5. Generate Stan code via bridge to existing make_stancode
  bridge <- make_stancode_bridge(model, dm, SZDG)
  stancode <- make_stancode(bridge, debug = .debug)

  # 6. Compile and run MCMC
  stanmodel <- rstan::stan_model(model_code = stancode)
  sampling_args <- list(
    object = stanmodel, data = standata,
    chains = chains, iter = iter, warmup = warmup, cores = cores
  )
  if (!is.null(seed)) sampling_args$seed <- seed
  stanfit <- do.call(rstan::sampling, c(sampling_args, list(...)))

  # 7. Return PStrataFit
  res <- list(
    call               = cl,
    model              = model,
    data               = data,
    stanfit            = stanfit,
    stancode           = stancode,
    standata           = standata,
    SZDG_table         = SZDG,
    strata_matrix      = strata_enc$strata_matrix,
    max_postrand_level = strata_enc$max_postrand_level,
    design_matrices    = list(XS = dm$XS, XG = dm$XG),
    time_points        = if (model$is_survival) standata$time else NULL
  )

  class(res) <- if (model$is_survival) {
    c("PStrataFitSurvival", "PStrataFit")
  } else {
    "PStrataFit"
  }

  res
}

# ===========================================================================
# Design matrix construction
# ===========================================================================

#' Build design matrices and extract response columns from data
#' @noRd
build_design_matrices <- function(model, data) {
  # Validate LHS columns exist
  needed <- c(model$treatment_name, model$postrand_names, model$outcome_name)
  if (!is.null(model$event_name)) needed <- c(needed, model$event_name)
  missing_cols <- setdiff(needed, names(data))
  if (length(missing_cols) > 0)
    stop("Columns not found in data: ", paste(missing_cols, collapse = ", "))

  # Validate RE grouping variables exist
  re_groups <- c(
    vapply(model$s_random_effects %||% list(), `[[`, character(1), "group"),
    vapply(model$y_random_effects %||% list(), `[[`, character(1), "group")
  )
  missing_re <- setdiff(re_groups, names(data))
  if (length(missing_re) > 0)
    stop("RE grouping variables not found in data: ",
         paste(missing_re, collapse = ", "))

  # Fixed-effect design matrices (strip response and random effects)
  s_terms <- stats::delete.response(
    stats::terms(reformulas::nobars(model$stratum_formula))
  )
  y_terms <- stats::delete.response(
    stats::terms(reformulas::nobars(model$outcome_formula))
  )
  XS <- stats::model.matrix(s_terms, data)
  XG <- stats::model.matrix(y_terms, data)

  # Extract response columns
  Z      <- data[[model$treatment_name]]
  D_cols <- lapply(model$postrand_names, function(d) data[[d]])
  Y      <- data[[model$outcome_name]]
  delta  <- if (!is.null(model$event_name)) data[[model$event_name]] else NULL

  # Random effect design matrices (via lme4)
  s_re <- parse_re_from_data(model$stratum_formula, model$s_random_effects, data)
  y_re <- parse_re_from_data(model$outcome_formula, model$y_random_effects, data)

  list(
    XS = XS, XG = XG,
    Z = Z, D_cols = D_cols, Y = Y, delta = delta,
    s_re = s_re, y_re = y_re
  )
}

#' Parse random effect design matrices from actual data using lme4
#' @noRd
parse_re_from_data <- function(formula, re_info, data) {
  if (is.null(re_info) || length(re_info) == 0) return(NULL)

  # Construct dummy-response formula for lme4
  rhs_str <- paste(deparse(formula[[3]], width.cutoff = 500), collapse = "")
  data$.y <- 0
  dummy_f <- stats::as.formula(paste(".y ~", rhs_str))

  lf <- lme4::lFormula(dummy_f, data)

  mapply(
    function(mat, cnms, fac) {
      list(matrix = t(as.matrix(mat)), terms = cnms, factors = fac)
    },
    lf$reTrms$Ztlist,
    lf$reTrms$cnms,
    lf$reTrms$flist,
    SIMPLIFY = FALSE
  )
}

# ===========================================================================
# Strata encoding
# ===========================================================================

#' Encode strata_def into integer strata_matrix for Stan
#'
#' Converts the list-of-lists strata definition into an integer matrix
#' where rows = strata, cols = treatment arms, values = encoded D integer.
#' Multiple D variables are encoded via polynomial: d1 + d2*(m1+1) + ...
#'
#' @noRd
encode_strata <- function(strata_def, num_treatment) {
  num_postrand <- length(strata_def[[1]])

  # Compute max level per postrand variable
  max_level <- integer(num_postrand)
  for (s in strata_def) {
    for (j in seq_len(num_postrand)) {
      max_level[j] <- max(max_level[j], max(s[[j]]))
    }
  }

  # Polynomial encoding bases
  bases <- c(1L, cumprod(max_level + 1L))

  mat <- matrix(0L, nrow = length(strata_def), ncol = num_treatment)
  for (i in seq_along(strata_def)) {
    for (z in seq_len(num_treatment)) {
      d_vals <- vapply(strata_def[[i]], function(v) v[z], integer(1))
      mat[i, z] <- as.integer(sum(c(d_vals, 0L) * bases))
    }
  }

  list(strata_matrix = mat, max_postrand_level = max_level)
}

#' Encode observed D columns from data into a single integer vector
#' @noRd
encode_D_data <- function(D_cols, max_postrand_level) {
  if (length(D_cols) == 1) return(as.integer(D_cols[[1]]))
  D_mat <- do.call(cbind, D_cols)
  bases <- c(1L, cumprod(max_postrand_level + 1L))
  apply(D_mat, 1, function(x) as.integer(sum(c(x, 0L) * bases)))
}

#' Encode treatment values to 0-based integers
#' @noRd
encode_Z_data <- function(Z, num_treatment) {
  if (is.factor(Z)) return(as.integer(Z) - 1L)
  z_int <- as.integer(Z)
  if (all(z_int %in% 0L:(num_treatment - 1L))) return(z_int)
  warning("Treatment variable auto-converted to 0-based integers.")
  as.integer(as.factor(Z)) - 1L
}

# ===========================================================================
# SZDG table construction
# ===========================================================================

#' Build the (Stratum, Treatment, D-encoded, outcome Group) mapping table
#'
#' Each row maps a combination of stratum and treatment arm to the observed
#' D value and the outcome parameter group. Under exclusion restriction (ER),
#' a stratum with identical D values across treatment arms shares one group.
#'
#' @noRd
build_SZDG <- function(strata_matrix, er_list, num_strata, num_treatment) {
  SZDG <- matrix(nrow = 0L, ncol = 4L)
  colnames(SZDG) <- c("S", "Z", "D", "G")
  G_id <- 0L

  for (s in seq_len(num_strata)) {
    S_id <- s - 1L
    for (z in seq_len(num_treatment)) {
      Z_id <- z - 1L
      D_id <- strata_matrix[s, z]

      # Under ER, reuse group if same stratum & D value already seen
      tmp_G <- NULL
      if (er_list[s] && nrow(SZDG) > 0) {
        match_idx <- which(SZDG[, "S"] == S_id & SZDG[, "D"] == D_id)
        if (length(match_idx) > 0) tmp_G <- SZDG[match_idx[1], "G"]
      }

      if (is.null(tmp_G)) {
        G_id <- G_id + 1L
        tmp_G <- G_id
      }
      SZDG <- rbind(SZDG, c(S_id, Z_id, D_id, tmp_G))
    }
  }

  SZDG
}

# ===========================================================================
# Stan data assembly
# ===========================================================================

#' Assemble the named list of data for Stan
#' @noRd
assemble_standata <- function(model, dm, strata_enc, SZDG) {
  Z_int <- encode_Z_data(dm$Z, model$num_treatment)
  D_int <- encode_D_data(dm$D_cols, strata_enc$max_postrand_level)

  sd <- list(
    N  = nrow(dm$XS),
    PS = ncol(dm$XS),
    PG = ncol(dm$XG),
    Z  = Z_int,
    D  = D_int,
    Y  = dm$Y,
    XS = dm$XS,
    XG = dm$XG
  )

  # Survival-specific fields
  if (model$is_survival) {
    sd$delta <- as.integer(dm$delta)
    stp <- model$survival_time_points
    sd$time <- if (length(stp) == 1) {
      seq(0, max(sd$Y) * 0.9, length.out = stp)
    } else {
      stp
    }
    sd[["T"]] <- length(sd$time)
  }

  # Random effect matrices
  sd <- append_re_standata(sd, dm$s_re, "S")
  sd <- append_re_standata(sd, dm$y_re, "G")

  sd
}

#' Append random effect data to the Stan data list
#' @noRd
append_re_standata <- function(sd, re_list, prefix) {
  if (is.null(re_list)) return(sd)
  for (i in seq_along(re_list)) {
    sd[[paste0("P", prefix, "_RE_", i)]] <- length(re_list[[i]]$terms)
    sd[[paste0("N", prefix, "_RE_", i)]] <- length(levels(re_list[[i]]$factors))
    sd[[paste0("X", prefix, "_RE_", i)]] <- re_list[[i]]$matrix
  }
  sd
}

# ===========================================================================
# Bridge to existing make_stancode
# ===========================================================================

#' Create a minimal object that the existing make_stancode / build_stan_context
#' can consume, mapping PStrataModel fields to the PSObject-like interface.
#' @noRd
make_stancode_bridge <- function(model, dm, SZDG) {
  safe <- function(x) if (is.null(x)) prior_flat() else x

  list(
    SZDG_table        = SZDG,
    Y.family          = model$outcome_family,
    S.formula         = list(random_eff_list = dm$s_re),
    Y.formula         = list(random_eff_list = dm$y_re),
    prior_intercept   = model$priors$intercept,
    prior_coefficient = model$priors$coefficient,
    prior_sigma       = safe(model$priors$sigma),
    prior_alpha       = safe(model$priors$alpha),
    prior_lambda      = safe(model$priors$lambda),
    prior_theta       = safe(model$priors$theta)
  )
}

# ===========================================================================
# Null-default operator (if not already available)
# ===========================================================================

if (!exists("%||%")) {
  `%||%` <- function(x, y) if (is.null(x)) y else x
}

# ===========================================================================
# PStrataFit: print
# ===========================================================================

#' @export
print.PStrataFit <- function(x, ...) {
  m <- x$model
  samples <- rstan::extract(x$stanfit)

  cat("PStrataFit\n")
  cat("  Family:", m$outcome_family$family,
      "(", m$outcome_family$link, ")\n")

  # Strata with ER markers
  strata_strs <- vapply(seq_along(m$strata_names), function(i) {
    if (m$exclusion_restriction[i]) paste0(m$strata_names[i], " (ER)")
    else m$strata_names[i]
  }, character(1))
  cat("  Strata:", paste(strata_strs, collapse = ", "), "\n")
  cat("  Outcome groups:", paste(m$outcome_groups$labels, collapse = ", "), "\n")

  cat("  N:", x$standata$N, "observations\n")

  si <- x$stanfit@sim
  cat("  MCMC:", si$chains, "chains,", si$iter, "iter",
      "(", si$warmup, "warmup )\n")

  # Stratum proportions
  strata_prob <- colMeans(samples$strata_prob)
  names(strata_prob) <- m$strata_names
  cat("\nEstimated stratum proportions:\n")
  print(round(strata_prob, 4))

  # Mean outcome per group (non-survival only)
  if ("mean_effect" %in% names(samples)) {
    mean_eff <- colMeans(as.matrix(samples$mean_effect))
    names(mean_eff) <- m$outcome_groups$labels
    cat("\nEstimated mean outcome per group:\n")
    print(round(mean_eff, 4))
  }

  cat("\nUse summary() for posterior intervals.\n")
  cat("Use diagnostics() to check MCMC convergence.\n")
  invisible(x)
}

#' @export
print.PStrataFitSurvival <- function(x, ...) {
  NextMethod()
  tp <- x$time_points
  if (!is.null(tp) && length(tp) > 1) {
    cat("Survival evaluated at", length(tp), "time points in [",
        round(min(tp), 2), ",", round(max(tp), 2), "].\n")
  }
}

# ===========================================================================
# PStrataFit: summary
# ===========================================================================

#' @export
summary.PStrataFit <- function(object,
                                probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
                                ...) {
  samples <- rstan::extract(object$stanfit)
  m <- object$model
  prob_names <- paste0(probs * 100, "%")

  # Stratum proportions
  sp <- samples$strata_prob
  sp_summary <- t(apply(sp, 2, function(x) {
    c(mean(x), stats::sd(x), stats::quantile(x, probs))
  }))
  rownames(sp_summary) <- m$strata_names
  colnames(sp_summary) <- c("mean", "sd", prob_names)

  # Mean outcome per group (non-survival)
  mean_effect_summary <- NULL
  if ("mean_effect" %in% names(samples)) {
    me <- as.matrix(samples$mean_effect)
    mean_effect_summary <- t(apply(me, 2, function(x) {
      c(mean(x), stats::sd(x), stats::quantile(x, probs))
    }))
    rownames(mean_effect_summary) <- m$outcome_groups$labels
    colnames(mean_effect_summary) <- c("mean", "sd", prob_names)
  }

  # beta_S (reference = first stratum)
  beta_S_summary <- summarize_coef_array(
    samples$beta_S,
    group_names = m$strata_names[-1],
    coef_names  = colnames(object$design_matrices$XS),
    probs       = probs
  )

  # beta_G
  beta_G_summary <- summarize_coef_array(
    samples$beta_G,
    group_names = m$outcome_groups$labels,
    coef_names  = colnames(object$design_matrices$XG),
    probs       = probs
  )

  # Extra parameters (sigma, theta, etc.)
  extra_summary <- list()
  for (ep in m$family_info$extra_params) {
    if (ep$name %in% names(samples)) {
      vals <- as.matrix(samples[[ep$name]])
      ep_sum <- t(apply(vals, 2, function(x) {
        c(mean(x), stats::sd(x), stats::quantile(x, probs))
      }))
      rownames(ep_sum) <- m$outcome_groups$labels
      colnames(ep_sum) <- c("mean", "sd", prob_names)
      extra_summary[[ep$name]] <- ep_sum
    }
  }

  res <- list(
    call          = object$call,
    model         = m,
    strata_prob   = sp_summary,
    mean_effect   = mean_effect_summary,
    beta_S        = beta_S_summary,
    beta_G        = beta_G_summary,
    extra_params  = extra_summary
  )
  class(res) <- "summary.PStrataFit"
  res
}

#' Summarize a 3D posterior coefficient array (iterations x groups x covariates)
#' @noRd
summarize_coef_array <- function(arr, group_names, coef_names, probs) {
  n_groups <- dim(arr)[2]
  n_coefs  <- dim(arr)[3]
  prob_names <- paste0(probs * 100, "%")

  rows   <- vector("list", n_groups * n_coefs)
  rnames <- character(n_groups * n_coefs)
  k <- 0L
  for (g in seq_len(n_groups)) {
    for (p in seq_len(n_coefs)) {
      k <- k + 1L
      x <- arr[, g, p]
      rows[[k]] <- c(mean(x), stats::sd(x), stats::quantile(x, probs))
      rnames[k] <- paste0(group_names[g], ":", coef_names[p])
    }
  }

  mat <- do.call(rbind, rows)
  rownames(mat) <- rnames
  colnames(mat) <- c("mean", "sd", prob_names)
  mat
}

#' @export
print.summary.PStrataFit <- function(x, digits = 4, ...) {
  cat("PStrataFit Summary\n")
  cat("====================\n\n")

  cat("Call:", deparse(x$call, width.cutoff = 200), "\n\n")

  cat("Stratum proportions:\n")
  print(round(x$strata_prob, digits))

  if (!is.null(x$mean_effect)) {
    cat("\nMean outcome per group:\n")
    print(round(x$mean_effect, digits))
  }

  cat("\nStratum model coefficients (beta_S, reference:",
      x$model$strata_names[1], "):\n")
  print(round(x$beta_S, digits))

  cat("\nOutcome model coefficients (beta_G):\n")
  print(round(x$beta_G, digits))

  for (nm in names(x$extra_params)) {
    cat("\n", nm, ":\n", sep = "")
    print(round(x$extra_params[[nm]], digits))
  }

  invisible(x)
}

# ===========================================================================
# PStrataFit: diagnostics
# ===========================================================================

#' MCMC Convergence Diagnostics
#'
#' @param object A \code{PStrataFit} object.
#' @param pars Character vector of parameter names. If NULL, key parameters
#'   are selected automatically.
#' @param ... Currently unused.
#' @return Invisibly returns the rstan summary matrix.
#' @export
diagnostics <- function(object, ...) UseMethod("diagnostics")

#' @rdname diagnostics
#' @export
diagnostics.PStrataFit <- function(object, pars = NULL, ...) {
  if (is.null(pars)) {
    pars <- c("beta_S", "beta_G", "strata_prob")
    all_pars <- names(rstan::extract(object$stanfit))
    extra <- intersect(c("sigma", "alpha", "lambda", "theta"), all_pars)
    pars <- c(pars, extra)
  }

  fit_summary <- rstan::summary(object$stanfit, pars = pars)$summary
  rhat  <- fit_summary[, "Rhat"]
  n_eff <- fit_summary[, "n_eff"]

  cat("MCMC Diagnostics\n")
  cat("----------------\n")
  cat("Rhat  - max:", round(max(rhat, na.rm = TRUE), 4),
      " min:", round(min(rhat, na.rm = TRUE), 4), "\n")
  cat("n_eff - min:", round(min(n_eff, na.rm = TRUE)),
      " median:", round(stats::median(n_eff, na.rm = TRUE)), "\n")

  if (any(rhat > 1.1, na.rm = TRUE)) {
    bad <- names(rhat[rhat > 1.1])
    cat("WARNING: Rhat > 1.1 for:",
        paste(utils::head(bad, 10), collapse = ", "),
        if (length(bad) > 10) "...", "\n")
  } else {
    cat("All Rhat values <= 1.1 (good).\n")
  }

  invisible(fit_summary)
}

# ===========================================================================
# PStrataFit: accessors
# ===========================================================================

#' @export
coef.PStrataFit <- function(object, ...) {
  samples <- rstan::extract(object$stanfit)
  result <- list()

  if ("beta_S" %in% names(samples)) {
    beta_S <- apply(samples$beta_S, c(2, 3), mean)
    rownames(beta_S) <- object$model$strata_names[-1]
    colnames(beta_S) <- colnames(object$design_matrices$XS)
    result$beta_S <- beta_S
  }

  if ("beta_G" %in% names(samples)) {
    beta_G <- apply(samples$beta_G, c(2, 3), mean)
    rownames(beta_G) <- object$model$outcome_groups$labels
    colnames(beta_G) <- colnames(object$design_matrices$XG)
    result$beta_G <- beta_G
  }

  for (p in c("sigma", "alpha", "lambda", "theta")) {
    if (p %in% names(samples)) {
      vals <- colMeans(as.matrix(samples[[p]]))
      names(vals) <- object$model$outcome_groups$labels
      result[[p]] <- vals
    }
  }

  result
}

#' @export
nobs.PStrataFit <- function(object, ...) object$standata$N

#' @export
formula.PStrataFit <- function(x, ...) {
  list(S = x$model$stratum_formula, Y = x$model$outcome_formula)
}

#' Extract the Generated Stan Code
#'
#' @param object A \code{PStrataFit} object.
#' @param ... Currently unused.
#' @return The Stan code as a character string.
#' @export
stancode.PStrataFit <- function(object, ...) object$stancode

# ===========================================================================
# PStrataFit: plot
# ===========================================================================

#' Plot PStrataFit Objects
#'
#' By default, shows violin plots of posterior strata proportions
#' and potential outcomes (non-survival).
#' Use \code{what = "trace"} or \code{what = "dens"} for MCMC diagnostics.
#'
#' @param x A \code{PStrataFit} object.
#' @param what \code{"default"} for strata proportions + potential outcomes,
#'   \code{"trace"} for traceplots, \code{"dens"} for density plots.
#' @param pars Parameter names for trace/dens plots (ignored for default).
#' @param ... Additional arguments passed to plotting functions.
#' @return A ggplot object.
#' @export
plot.PStrataFit <- function(x, what = c("default", "trace", "dens"),
                             pars = "strata_prob", ...) {
  what <- match.arg(what)
  if (what == "trace") return(rstan::traceplot(x$stanfit, pars = pars, ...))
  if (what == "dens") return(rstan::stan_dens(x$stanfit, pars = pars, ...))

  m <- x$model
  samples <- rstan::extract(x$stanfit)

  # --- Strata proportions: [iter, S] → long ---
  sp <- samples$strata_prob
  sp_df <- data.frame(
    label = paste0("P(", rep(m$strata_names, each = nrow(sp)), ")"),
    value = as.vector(sp),
    stringsAsFactors = FALSE
  )

  # --- Potential outcomes (non-survival) ---
  if (!m$is_survival && "mean_effect" %in% names(samples)) {
    est <- estimate(x)
    arr <- est$outcome_array  # [S, Z, iter]
    dims <- dim(arr)
    dn <- dimnames(arr)

    po_grid <- expand.grid(
      iter = seq_len(dims[3]),
      Z    = dn[[2]],
      S    = dn[[1]],
      stringsAsFactors = FALSE
    )
    po_df <- data.frame(
      label = paste0("E[Y|", po_grid$S, ",", po_grid$Z, "]"),
      value = as.vector(aperm(arr, c(3, 2, 1))),
      stringsAsFactors = FALSE
    )
    plot_df <- rbind(sp_df, po_df)
  } else {
    plot_df <- sp_df
  }
  plot_df$label <- factor(plot_df$label, levels = unique(plot_df$label))

  ggplot2::ggplot(plot_df, ggplot2::aes(x = value)) +
    ggplot2::geom_density(fill = "steelblue", alpha = 0.5) +
    ggplot2::facet_wrap(~ label, scales = "free") +
    ggplot2::labs(x = NULL, y = NULL) +
    ggplot2::theme_minimal() +
    ggplot2::theme(axis.text.y  = ggplot2::element_blank(),
                   axis.ticks.y = ggplot2::element_blank())
}

# ===========================================================================
# contrast.R — Compute causal contrasts from PSEstimate
# ===========================================================================

#' Contrast Potential Outcomes
#'
#' Compute pairwise differences of posterior potential outcomes
#' along strata, treatment arms, or time points.
#'
#' @param object A \code{PSEstimate}, \code{PSContrast}, or
#'   \code{PStrataFit} object. If \code{PStrataFit}, \code{estimate()}
#'   is called automatically.
#' @param S Strata to contrast. \code{TRUE} for all pairwise, or a
#'   numeric/logical index vector. \code{NULL} (default) = no contrast.
#' @param Z Treatment arms to contrast. \code{TRUE} for all pairwise.
#' @param T Time points to contrast (survival only).
#' @param type \code{"all"} (all pairwise), \code{"sequential"}
#'   (consecutive pairs), or \code{"cycle"} (consecutive + wrap-around).
#' @param ... Additional arguments passed to \code{estimate()} when
#'   \code{object} is a \code{PStrataFit}.
#' @return A \code{PSContrast} object (inherits from \code{PSEstimate}).
#' @examples
#' \donttest{
#' data(sim_data_normal)
#' model <- PStrataModel(
#'   S.formula = Z + D ~ 1,
#'   Y.formula = Y ~ 1,
#'   Y.family  = gaussian(),
#'   strata    = c(n = "00", c = "01", a = "11"),
#'   ER        = c("n", "a")
#' )
#' ps_fit <- fit(model, data = sim_data_normal, chains = 2, iter = 500)
#'
#' # Treatment effect (Z contrast) for each stratum
#' ctr_z <- contrast(ps_fit, Z = TRUE)
#' summary(ctr_z)
#' plot(ctr_z)
#'
#' # Stratum contrasts for each treatment arm
#' ctr_s <- contrast(ps_fit, S = TRUE)
#' summary(ctr_s)
#' }
#' @export
contrast <- function(object, ...) UseMethod("contrast")

#' @rdname contrast
#' @export
contrast.PSEstimate <- function(object, S = NULL, Z = NULL, T = NULL,
                                type = c("all", "sequential", "cycle"),
                                ...) {
  type <- match.arg(type)
  arr <- object$outcome_array

  if (!is.null(S)) arr <- contrast_along_dim(arr, 1L, S, type)
  if (!is.null(Z)) arr <- contrast_along_dim(arr, 2L, Z, type)
  if (!is.null(T) && object$is_survival)
    arr <- contrast_along_dim(arr, 3L, T, type)

  structure(list(
    outcome_array = arr,
    is_survival   = object$is_survival,
    time_points   = object$time_points,
    model_info    = object$model_info
  ), class = c("PSContrast", "PSEstimate"))
}

#' @rdname contrast
#' @export
contrast.PStrataFit <- function(object, S = NULL, Z = NULL, T = NULL,
                                type = c("all", "sequential", "cycle"),
                                ...) {
  est <- estimate(object, ...)
  contrast(est, S = S, Z = Z, T = T, type = type)
}

# ===========================================================================
# PSContrast: print
# ===========================================================================

#' @export
print.PSContrast <- function(x, ...) {
  dims <- dim(x$outcome_array)

  if (!x$is_survival) {
    cat("PSContrast (", dims[1], " x ", dims[2], " x ",
        dims[3], " draws)\n", sep = "")
  } else {
    cat("PSContrast [survival] (", dims[1], " x ", dims[2],
        " x ", dims[3], " time points x ", dims[4],
        " draws)\n", sep = "")
  }

  # Show compact summary: mean and 95% CI
  summary_arr <- summarize_last_dim(x$outcome_array)
  ndim <- length(dim(summary_arr))
  dimnames(summary_arr)[[ndim]] <- summary_stat_names()

  mat <- summary_array_to_matrix(summary_arr)
  show_cols <- c("mean", "2.5%", "97.5%")
  cat("\nPosterior summary:\n")
  print(round(mat[, show_cols, drop = FALSE], 4))

  cat("\nUse summary() for full posterior intervals.\n")
  invisible(x)
}

# ===========================================================================
# PSContrast: plot
# ===========================================================================

#' @export
plot.PSContrast <- function(x, ...) {
  arr <- x$outcome_array
  dims <- dim(arr)
  dn <- dimnames(arr)

  if (!x$is_survival) {
    df <- expand.grid(
      iter = seq_len(dims[3]),
      Z    = dn[[2]],
      S    = dn[[1]],
      stringsAsFactors = FALSE
    )
    df$value <- as.vector(aperm(arr, c(3, 2, 1)))
    df$S <- factor(df$S, levels = dn[[1]])

    if (dims[2] == 1) {
      df$label <- paste0(df$S, ": ", dn[[2]])
    } else {
      df$label <- paste0(df$S, ": ", df$Z)
    }
    df$label <- factor(df$label, levels = unique(df$label))

    ggplot2::ggplot(df, ggplot2::aes(x = value)) +
      ggplot2::geom_density(fill = "steelblue", alpha = 0.5) +
      ggplot2::geom_vline(xintercept = 0, linetype = "dashed",
                          color = "red") +
      ggplot2::facet_wrap(~ label, scales = "free") +
      ggplot2::labs(x = NULL, y = NULL) +
      ggplot2::theme_minimal() +
      ggplot2::theme(axis.text.y  = ggplot2::element_blank(),
                     axis.ticks.y = ggplot2::element_blank())
  } else {
    # Survival contrast: ribbon plot with reference line at 0
    plot_df <- summary(x, "data.frame")
    plot_df$T <- as.numeric(plot_df$T)

    ggplot2::ggplot(plot_df,
        ggplot2::aes(x = T, y = mean)) +
      ggplot2::geom_line() +
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = `2.5%`, ymax = `97.5%`),
        alpha = 0.2) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed",
                          color = "red") +
      ggplot2::facet_wrap(~ S) +
      ggplot2::labs(x = "Time", y = "Contrast") +
      ggplot2::theme_minimal()
  }
}

# ===========================================================================
# estimate.R — Extract posterior potential outcomes from PStrataFit
# ===========================================================================

#' Extract Posterior Potential Outcomes
#'
#' Maps posterior samples of outcome-group means back to
#' the (stratum x treatment) grid via the SZDG table.
#'
#' @param object A \code{PStrataFit} object.
#' @param type For survival models: \code{"probability"} (survival probability)
#'   or \code{"RACE"} (restricted average causal effect).
#' @param ... Additional arguments (unused).
#' @return A \code{PSEstimate} object containing
#' \item{outcome_array}{[S, Z, iter] array for non-survival,
#'   or [S, Z, T, iter] for survival.}
#' \item{is_survival}{Logical.}
#' \item{time_points}{Numeric vector (survival only).}
#' \item{model_info}{List of strata names, treatment info,
#'   family, ER flags, outcome groups.}
#' @examples
#' \donttest{
#' data(sim_data_normal)
#' model <- PStrataModel(
#'   S.formula = Z + D ~ 1,
#'   Y.formula = Y ~ 1,
#'   Y.family  = gaussian(),
#'   strata    = c(n = "00", c = "01", a = "11"),
#'   ER        = c("n", "a")
#' )
#' ps_fit <- fit(model, data = sim_data_normal, chains = 2, iter = 500)
#' est <- estimate(ps_fit)
#' summary(est)
#' plot(est)
#' }
#' @export
estimate <- function(object, ...) UseMethod("estimate")

#' @rdname estimate
#' @export
estimate.PStrataFit <- function(object, type = c("probability", "RACE"), ...) {
  m <- object$model
  SZDG <- object$SZDG_table
  samples <- rstan::extract(object$stanfit)

  S_count <- m$num_strata
  Z_count <- m$num_treatment
  Z_names <- paste0(m$treatment_name, "=", 0:(Z_count - 1))

  mi <- list(
    strata_names          = m$strata_names,
    treatment_name        = m$treatment_name,
    Z_names               = Z_names,
    family                = m$outcome_family,
    exclusion_restriction = m$exclusion_restriction,
    outcome_groups        = m$outcome_groups
  )

  if (!m$is_survival) {
    mean_effect <- samples$mean_effect
    arr <- .map_SZDG_to_array(mean_effect, SZDG, S_count, Z_count)
    dimnames(arr) <- list(S = m$strata_names, Z = Z_names, iter = NULL)

    return(structure(list(
      outcome_array = arr,
      is_survival   = FALSE,
      time_points   = NULL,
      model_info    = mi
    ), class = "PSEstimate"))
  }

  # --- survival ---
  type <- match.arg(type)
  extract_name <- if (type == "probability") "mean_surv_prob" else "mean_RACE"
  mean_outcome <- samples[[extract_name]]
  T_count    <- dim(mean_outcome)[3]
  iter_count <- dim(mean_outcome)[1]

  arr <- array(NA, dim = c(S_count, Z_count, T_count, iter_count))
  for (s in seq_len(S_count)) {
    for (z in seq_len(Z_count)) {
      G <- SZDG[SZDG[, "S"] == s - 1 & SZDG[, "Z"] == z - 1, "G"]
      arr[s, z, , ] <- t(mean_outcome[, G, ])
    }
  }
  dimnames(arr) <- list(
    S = m$strata_names, Z = Z_names,
    T = seq_len(T_count), iter = NULL
  )

  structure(list(
    outcome_array = arr,
    is_survival   = TRUE,
    time_points   = object$time_points,
    type          = type,
    model_info    = mi
  ), class = c("PSEstimateSurvival", "PSEstimate"))
}

# --- internal helper ---

#' Map mean_effect[iter, G] to [S, Z, iter] via SZDG
#' @noRd
.map_SZDG_to_array <- function(mean_effect, SZDG, S_count, Z_count) {
  iter_count <- nrow(mean_effect)
  arr <- array(NA, dim = c(S_count, Z_count, iter_count))
  for (s in seq_len(S_count)) {
    for (z in seq_len(Z_count)) {
      G <- SZDG[SZDG[, "S"] == s - 1 & SZDG[, "Z"] == z - 1, "G"]
      arr[s, z, ] <- mean_effect[, G]
    }
  }
  arr
}

# ===========================================================================
# PSEstimate: print
# ===========================================================================

#' @export
print.PSEstimate <- function(x, ...) {
  dims <- dim(x$outcome_array)
  mi <- x$model_info

  if (!x$is_survival) {
    cat("PSEstimate (", dims[1], " strata x ", dims[2],
        " treatments x ", dims[3], " draws)\n", sep = "")
  } else {
    cat("PSEstimate [survival] (", dims[1], " strata x ", dims[2],
        " treatments x ", dims[3], " time points x ", dims[4],
        " draws)\n", sep = "")
  }

  # Posterior mean table (non-survival only)
  if (!x$is_survival) {
    summary_arr <- summarize_last_dim(x$outcome_array)
    mean_mat <- summary_arr[, , 1, drop = TRUE]
    if (is.null(dim(mean_mat)))
      mean_mat <- matrix(mean_mat, nrow = dims[1], ncol = dims[2])
    rownames(mean_mat) <- mi$strata_names
    colnames(mean_mat) <- mi$Z_names

    cat("\nPosterior mean E[Y(z) | S=s]:\n")
    print(round(mean_mat, 4))
  }

  cat("\nUse summary() for posterior intervals.\n")
  cat("Use contrast() to compute causal effects.\n")
  invisible(x)
}

# ===========================================================================
# PSEstimate: summary
# ===========================================================================

#' @export
summary.PSEstimate <- function(object,
                               type = c("array", "matrix", "data.frame"),
                               ...) {
  type <- match.arg(type)

  summary_array <- summarize_last_dim(object$outcome_array)
  ndim <- length(dim(summary_array))
  dimnames(summary_array)[[ndim]] <- summary_stat_names()

  if (type == "array") return(summary_array)

  mat <- summary_array_to_matrix(summary_array)
  if (type == "matrix") return(mat)

  col_names <- if (object$is_survival) c("S", "Z", "T") else c("S", "Z")
  summary_matrix_to_df(mat, col_names, object$time_points)
}

# ===========================================================================
# PSEstimate: plot
# ===========================================================================

#' @export
plot.PSEstimate <- function(x, ...) {
  mi <- x$model_info

  if (!x$is_survival) {
    arr <- x$outcome_array  # [S, Z, iter]
    dims <- dim(arr)
    dn <- dimnames(arr)

    df <- expand.grid(
      iter = seq_len(dims[3]),
      Z    = dn[[2]],
      S    = dn[[1]],
      stringsAsFactors = FALSE
    )
    df$value <- as.vector(aperm(arr, c(3, 2, 1)))
    df$S <- factor(df$S, levels = dn[[1]])

    df$label <- paste0("E[Y|", df$S, ",", df$Z, "]")
    df$label <- factor(df$label, levels = unique(df$label))

    ggplot2::ggplot(df, ggplot2::aes(x = value)) +
      ggplot2::geom_density(fill = "steelblue", alpha = 0.5) +
      ggplot2::facet_wrap(~ label, scales = "free") +
      ggplot2::labs(x = NULL, y = NULL) +
      ggplot2::theme_minimal() +
      ggplot2::theme(axis.text.y  = ggplot2::element_blank(),
                     axis.ticks.y = ggplot2::element_blank())

  } else {
    # Survival: ribbon plot over time, faceted by stratum
    plot_df <- summary(x, "data.frame")
    plot_df$T <- as.numeric(plot_df$T)

    ggplot2::ggplot(plot_df,
        ggplot2::aes(x = T, y = mean, color = Z, fill = Z)) +
      ggplot2::geom_line() +
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = `2.5%`, ymax = `97.5%`),
        alpha = 0.2, color = NA) +
      ggplot2::facet_wrap(~ S) +
      ggplot2::labs(x = "Time", y = "Survival probability",
                    color = "Treatment", fill = "Treatment") +
      ggplot2::theme_minimal()
  }
}

# ===========================================================================
# PSEstimate: subsetting
# ===========================================================================

#' @export
`[.PSEstimate` <- function(x, S, Z, T, iter) {
  if (missing(S)) S <- TRUE
  if (missing(Z)) Z <- TRUE
  if (missing(T)) T <- TRUE
  if (missing(iter)) iter <- TRUE

  if (!x$is_survival) {
    if (!isTRUE(iter))
      x$outcome_array <- x$outcome_array[S, Z, iter, drop = FALSE]
    else
      x$outcome_array <- x$outcome_array[S, Z, T, drop = FALSE]
  } else {
    x$outcome_array <- x$outcome_array[S, Z, T, iter, drop = FALSE]
  }

  x
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{fit}
\alias{fit}
\alias{fit.PStrataModel}
\title{Fit a Principal Stratification Model}
\usage{
fit(model, ...)

\method{fit}{PStrataModel}(
  model,
  data,
  chains = 4,
  iter = 2000,
  warmup = floor(iter/2),
  cores = 1,
  seed = NULL,
  .debug = FALSE,
  ...
)
}
\arguments{
\item{model}{A \code{\link{PStrataModel}} object.}

\item{...}{Additional arguments passed to \code{\link[rstan]{stan}}.}

\item{data}{A data frame containing all variables referenced in the model.}

\item{chains, iter, warmup, cores, seed}{MCMC settings passed to
\code{\link[rstan]{stan}}.}

\item{.debug}{Logical. If TRUE, read Stan helper files from \code{inst/}
rather than installed package data. Useful during development.}
}
\value{
An object of class \code{PStrataFit}
  (or \code{PStrataFitSurvival}).
}
\description{
Estimate a principal stratification model by running MCMC via Stan.
The model specification (a \code{\link{PStrataModel}}) is combined with
observed data to produce posterior samples.
}
\examples{
\donttest{
data(sim_data_normal)
model <- PStrataModel(
  S.formula = Z + D ~ 1,
  Y.formula = Y ~ 1,
  Y.family  = gaussian(),
  strata    = c(n = "00", c = "01", a = "11"),
  ER        = c("n", "a")
)
ps_fit <- fit(model, data = sim_data_normal, chains = 2, iter = 500)
summary(ps_fit)
diagnostics(ps_fit)
plot(ps_fit)
cat(ps_fit$stancode)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_stancode.R
\name{make_stancode}
\alias{make_stancode}
\title{Stan Code for PStrata Models}
\usage{
make_stancode(object, filename = NULL, debug = FALSE)
}
\arguments{
\item{object}{An internal model specification object.}

\item{filename}{(optional) string. If not \code{NULL}, the
Stan file is saved to this path.}

\item{debug}{only for development/testing use.}
}
\value{
A string containing the Stan program.
}
\description{
Generate the Stan code corresponding to the model.
This is called internally by \code{\link{fit}}; use
\code{\link{stancode.PStrataFit}} to retrieve the code
from a fitted model.
}
\examples{
\donttest{
data(sim_data_normal)
model <- PStrataModel(
  S.formula = Z + D ~ 1,
  Y.formula = Y ~ 1,
  Y.family  = gaussian(),
  strata    = c(n = "00", c = "01", a = "11"),
  ER        = c("n", "a")
)
ps_fit <- fit(model, data = sim_data_normal, chains = 2, iter = 500)
cat(ps_fit$stancode)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_data_normal.R
\docType{data}
\name{sim_data_normal}
\alias{sim_data_normal}
\title{Simulated Dataset for Normal Outcome}
\format{
## `sim_data_normal`
A data frame with 1,000 rows and 4 columns:
\describe{
  \item{S}{Principal Strata: "never taker", "complier" or "always taker"}
  \item{Z}{Randomized treatment arm: 0 = control, 1 = treatment}
  \item{D}{Actual treatment arm: 0 = control, 1 = treatment}
  \item{Y}{Outcome}
}
}
\usage{
sim_data_normal
}
\description{
A dataset generated for illustration of the principal stratification analysis.
This dataset represents the common case of non-compliance.
}
\details{
The dataset represents the scenario where actual treatment might not be in compliance
with the randomized (assigned) treatment. Defiers are ruled out, leaving three
strata, "never taker", "complier" and "always taker" randomly sampled with 
probability 0.3, 0.2 and 0.5 respectively. The assigned treatment \eqn{Z} is randomized
with 0.5 probability for either arm. The outcome \eqn{Y} is given by the following.
\describe{
\item{never taker}{\eqn{Y \sim N(3, 1)}}
\item{complier}{\eqn{Y \sim N(-1-Z, 0.5)}}
\item{always taker}{\eqn{Y \sim N(1, 2)}}
}
The exclusion restriction assumption holds for never takers and always takers in this
generated dataset.
}
\examples{
data(sim_data_normal)
head(sim_data_normal)
table(sim_data_normal$D, sim_data_normal$Z)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival.R
\name{survival}
\alias{survival}
\title{Family Function for Survival Data}
\usage{
survival(method = c("Cox", "AFT"), link = "identity")
}
\arguments{
\item{method}{the parametric method used for survival data. Can be Cox or AFT.}

\item{link}{a link function, currently only identity is implemented and used}
}
\value{
A \code{family} object
}
\description{
Construct a family object for survival data
}
\examples{
survival("Cox")
survival("AFT")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{plot.PStrataFit}
\alias{plot.PStrataFit}
\title{Plot PStrataFit Objects}
\usage{
\method{plot}{PStrataFit}(x, what = c("default", "trace", "dens"), pars = "strata_prob", ...)
}
\arguments{
\item{x}{A \code{PStrataFit} object.}

\item{what}{\code{"default"} for strata proportions + potential outcomes,
\code{"trace"} for traceplots, \code{"dens"} for density plots.}

\item{pars}{Parameter names for trace/dens plots (ignored for default).}

\item{...}{Additional arguments passed to plotting functions.}
}
\value{
A ggplot object.
}
\description{
By default, shows violin plots of posterior strata proportions
and potential outcomes (non-survival).
Use \code{what = "trace"} or \code{what = "dens"} for MCMC diagnostics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PStrata-package.R
\name{PStrata_PACKAGE}
\alias{PStrata_PACKAGE}
\title{PStrata: Principal STRATification Analysis for Data with Post-Randomization Confounding}
\description{
The \bold{PStrata} package is designed for estimating causal effects in the presence of post-treatment confounding using principal stratification.
It provides an interface to fit the Bayesian principal stratification model, which is a complex mixture model, using \bold{Stan}, a C++ package for obtaining full Bayesian inference.
The formula syntax is an extended version of the syntax applied in many regression functions and packages, such as \code{\link{lm}}, \code{\link{glm}} and \pkg{\link[lme4]{lme4-package}}, to provide a simple interface.
A wide variety of distributions and link functions are supported, allowing users to fit linear, binary or count data, and survival models with principal stratification.
Further modeling options include multiple post-treatment confounding variables and cluster random effects.
The monotonicity and exclusion restriction assumptions can be easily applied, and
prior specifications are flexible and encourage users to reflect their prior belief.
In addition, all parameters can be inferred from the posterior distribution, which enables further analysis other than provided by the package.
}
\details{
The Bayesian principal stratification analysis relies on two models, the principal stratum model and the outcome model.
The main workflow is:
\enumerate{
  \item \code{\link{PStrataModel}}: specify the model (formulas, strata, priors).
  \item \code{\link{fit}}: compile and run MCMC sampling via Stan.
  \item \code{\link{estimate}}: extract posterior potential outcomes.
  \item \code{\link{contrast}}: compute causal effect contrasts.
}
Based on the supplied formulas, data and additional information,
it automatically generates the Stan code via \code{\link{make_stancode}} and fits the model using \pkg{Stan}.

The estimated probability for each principal stratum and the estimated mean response are calculated with \pkg{Stan} as it is faster and more space-efficient.
Post-processing methods \code{\link{summary}} and \code{\link{plot}} provide overviews and visualization.

Because \pkg{PStrata} heavily relies on \pkg{Stan} for posterior sampling, a C++ compiler is required.
The program \pkg{Rtools} (available on \url{https://cran.r-project.org/bin/windows/Rtools/}) comes with a C++ compiler for Windows.
On Mac, Xcode is suggested. For further instructions on how to get the compilers running, please refer to the prerequisites section at the \href{https://github.com/stan-dev/rstan/wiki/RStan-Getting-Started}{RStan-Getting-Started} page.
}
\references{
The Stan Development Team. Stan Modeling Language User's Guide and Reference Manual. \url{https://mc-stan.org/users/documentation/}

Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. \url{https://mc-stan.org/}
}
\seealso{
\code{\link{PStrataModel}}, \code{\link{fit}}, \code{\link{estimate}}, \code{\link{contrast}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PStrataModel.R
\name{PStrataModel}
\alias{PStrataModel}
\title{Define a Principal Stratification Model}
\usage{
PStrataModel(
  S.formula,
  Y.formula,
  Y.family,
  strata,
  ER = NULL,
  prior_intercept = prior_flat(),
  prior_coefficient = prior_normal(),
  prior_sigma = prior_inv_gamma(),
  prior_alpha = prior_inv_gamma(),
  prior_lambda = prior_inv_gamma(),
  prior_theta = prior_normal(),
  survival.time.points = 50
)
}
\arguments{
\item{S.formula}{formula for the stratum model (e.g., \code{Z + D ~ X1 + X2}).}

\item{Y.formula}{formula for the outcome model (e.g., \code{Y ~ X1 + X2}).}

\item{Y.family}{a \code{family} object (e.g., \code{gaussian()}, \code{survival("Cox")}).}

\item{strata}{strata definition: character vector, list of vectors, or list of lists.}

\item{ER}{exclusion restriction: character vector of strata names or logical vector.}

\item{prior_intercept, prior_coefficient, prior_sigma, prior_alpha, prior_lambda, prior_theta}{prior distributions for model parameters.}

\item{survival.time.points}{number of time points for survival outcomes.}
}
\value{
An object of class \code{PStrataModel}.
}
\description{
Creates a model specification for principal stratification analysis.
No data is required at this stage -- the specification is purely symbolic.
Use \code{\link{fit}} to estimate the model with data.
}
\examples{
# Non-compliance with three strata (never-taker, complier, always-taker)
model <- PStrataModel(
  S.formula = Z + D ~ 1,
  Y.formula = Y ~ 1,
  Y.family  = gaussian(),
  strata    = c(n = "00", c = "01", a = "11"),
  ER        = c("n", "a")
)
print(model)
summary(model)

\donttest{
# Fit the model (requires rstan and C++ compiler)
data(sim_data_normal)
ps_fit <- fit(model, data = sim_data_normal, chains = 2, iter = 500)
summary(ps_fit)
plot(ps_fit)

# Extract potential outcomes and contrasts
est <- estimate(ps_fit)
summary(est)
plot(est)

ctr <- contrast(ps_fit)
summary(ctr)
plot(ctr)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{diagnostics}
\alias{diagnostics}
\alias{diagnostics.PStrataFit}
\title{MCMC Convergence Diagnostics}
\usage{
diagnostics(object, ...)

\method{diagnostics}{PStrataFit}(object, pars = NULL, ...)
}
\arguments{
\item{object}{A \code{PStrataFit} object.}

\item{...}{Currently unused.}

\item{pars}{Character vector of parameter names. If NULL, key parameters
are selected automatically.}
}
\value{
Invisibly returns the rstan summary matrix.
}
\description{
MCMC Convergence Diagnostics
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate.R
\name{estimate}
\alias{estimate}
\alias{estimate.PStrataFit}
\title{Extract Posterior Potential Outcomes}
\usage{
estimate(object, ...)

\method{estimate}{PStrataFit}(object, type = c("probability", "RACE"), ...)
}
\arguments{
\item{object}{A \code{PStrataFit} object.}

\item{...}{Additional arguments (unused).}

\item{type}{For survival models: \code{"probability"} (survival probability)
or \code{"RACE"} (restricted average causal effect).}
}
\value{
A \code{PSEstimate} object containing
\item{outcome_array}{[S, Z, iter] array for non-survival,
  or [S, Z, T, iter] for survival.}
\item{is_survival}{Logical.}
\item{time_points}{Numeric vector (survival only).}
\item{model_info}{List of strata names, treatment info,
  family, ER flags, outcome groups.}
}
\description{
Maps posterior samples of outcome-group means back to
the (stratum x treatment) grid via the SZDG table.
}
\examples{
\donttest{
data(sim_data_normal)
model <- PStrataModel(
  S.formula = Z + D ~ 1,
  Y.formula = Y ~ 1,
  Y.family  = gaussian(),
  strata    = c(n = "00", c = "01", a = "11"),
  ER        = c("n", "a")
)
ps_fit <- fit(model, data = sim_data_normal, chains = 2, iter = 500)
est <- estimate(ps_fit)
summary(est)
plot(est)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior.R
\name{prior}
\alias{prior}
\alias{prior_flat}
\alias{prior_normal}
\alias{prior_t}
\alias{prior_cauchy}
\alias{prior_lasso}
\alias{prior_logistic}
\alias{prior_chisq}
\alias{prior_inv_chisq}
\alias{prior_exponential}
\alias{prior_gamma}
\alias{prior_inv_gamma}
\alias{prior_weibull}
\title{Prior Distribution Functions}
\usage{
prior_flat()

prior_normal(mu = 0, sigma = 1)

prior_t(mu = 0, sigma = 1, df = 1)

prior_cauchy(mu = 0, sigma = 1)

prior_lasso(mu = 0, sigma = 1)

prior_logistic(mu = 0, sigma = 1)

prior_chisq(df = 1)

prior_inv_chisq(df = 1)

prior_exponential(beta = 1)

prior_gamma(alpha = 1, beta = 1)

prior_inv_gamma(alpha = 1, beta = 1)

prior_weibull(alpha = 1, sigma = 1)
}
\arguments{
\item{mu, sigma, df, alpha, beta}{parameters for the prior distribution}
}
\value{
A list, including the following items.
\describe{
\item{name}{name of the distribution}
\item{type}{type of the distribution, one character string of "real" or "positive"}
\item{args}{a named list of all the input parameters}
\item{call}{a function call object of the prior distribution on the parameters}
}
}
\description{
Define prior functions used in \code{PStrata}.
}
\examples{
prior_flat()
prior_normal(0, 10)
prior_t(0, 2.5, df = 3)
prior_cauchy(0, 5)
prior_inv_gamma(2, 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{stancode.PStrataFit}
\alias{stancode.PStrataFit}
\title{Extract the Generated Stan Code}
\usage{
stancode.PStrataFit(object, ...)
}
\arguments{
\item{object}{A \code{PStrataFit} object.}

\item{...}{Currently unused.}
}
\value{
The Stan code as a character string.
}
\description{
Extract the Generated Stan Code
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contrast.R
\name{contrast}
\alias{contrast}
\alias{contrast.PSEstimate}
\alias{contrast.PStrataFit}
\title{Contrast Potential Outcomes}
\usage{
contrast(object, ...)

\method{contrast}{PSEstimate}(
  object,
  S = NULL,
  Z = NULL,
  T = NULL,
  type = c("all", "sequential", "cycle"),
  ...
)

\method{contrast}{PStrataFit}(
  object,
  S = NULL,
  Z = NULL,
  T = NULL,
  type = c("all", "sequential", "cycle"),
  ...
)
}
\arguments{
\item{object}{A \code{PSEstimate}, \code{PSContrast}, or
\code{PStrataFit} object. If \code{PStrataFit}, \code{estimate()}
is called automatically.}

\item{...}{Additional arguments passed to \code{estimate()} when
\code{object} is a \code{PStrataFit}.}

\item{S}{Strata to contrast. \code{TRUE} for all pairwise, or a
numeric/logical index vector. \code{NULL} (default) = no contrast.}

\item{Z}{Treatment arms to contrast. \code{TRUE} for all pairwise.}

\item{T}{Time points to contrast (survival only).}

\item{type}{\code{"all"} (all pairwise), \code{"sequential"}
(consecutive pairs), or \code{"cycle"} (consecutive + wrap-around).}
}
\value{
A \code{PSContrast} object (inherits from \code{PSEstimate}).
}
\description{
Compute pairwise differences of posterior potential outcomes
along strata, treatment arms, or time points.
}
\examples{
\donttest{
data(sim_data_normal)
model <- PStrataModel(
  S.formula = Z + D ~ 1,
  Y.formula = Y ~ 1,
  Y.family  = gaussian(),
  strata    = c(n = "00", c = "01", a = "11"),
  ER        = c("n", "a")
)
ps_fit <- fit(model, data = sim_data_normal, chains = 2, iter = 500)

# Treatment effect (Z contrast) for each stratum
ctr_z <- contrast(ps_fit, Z = TRUE)
summary(ctr_z)
plot(ctr_z)

# Stratum contrasts for each treatment arm
ctr_s <- contrast(ps_fit, S = TRUE)
summary(ctr_s)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_data_Cox.R
\docType{data}
\name{sim_data_Cox}
\alias{sim_data_Cox}
\title{Simulated Dataset for Survival Outcome (Cox Model)}
\format{
## `sim_data_Cox`
A data frame with 1,000 rows and 7 columns:
\describe{
  \item{S}{Principal Strata: "never taker", "complier" or "always taker"}
  \item{Z}{Randomized treatment arm: 0 = control, 1 = treatment}
  \item{D}{Actual treatment arm: 0 = control, 1 = treatment}
  \item{T}{True outcome: event time}
  \item{C}{Censor time}
  \item{delta}{Event indicator. 1 means true outcome is observed; 0 means otherwise}
  \item{Y}{The observed event time or censor time}
}
}
\usage{
sim_data_Cox
}
\description{
A dataset generated for illustration of the principal stratification analysis.
This dataset represents the common case of non-compliance.
}
\details{
The dataset represents the scenario where actual treatment might not be in compliance
with the randomized (assigned) treatment. Defiers and always-takers are ruled out, leaving two
strata, "never-taker" and "complier" randomly sampled with 
probability 0.3, 0.7 respectively. The assigned treatment \eqn{Z} is randomized
with 0.5 probability for either arm. The true event time \eqn{T} is given by the following 
Weibull-Cox distribution
\describe{
\item{never-taker}{\eqn{Y \sim Weibull-Cox(theta = 1, mu = 0.3)}}
\item{complier}{\eqn{Y \sim Weibull-Cox(theta = 1, mu = 2 - 0.6*Z)}}
} and the censor time \eqn{C} is uniformly drawn between 0.5 and 2.

The exclusion restriction assumption holds for never-takers in this
generated dataset.
}
\examples{
data(sim_data_Cox)
head(sim_data_Cox)
table(sim_data_Cox$D, sim_data_Cox$Z)
}
\keyword{datasets}
